# -*- encoding: utf-8 -*-
from __future__ import with_statement, absolute_import

import datetime
import decimal
import os
import pickle
from threading import local

from django.conf import settings
from django.template import Template, Context
from django.template.base import TemplateSyntaxError
from django.test import TestCase, RequestFactory
from django.test.utils import override_settings
from django.utils import translation
from django.utils.formats import (get_format, date_format, time_format,
    localize, localize_input, iter_format_modules, get_format_modules,
    number_format)
from django.utils.importlib import import_module
from django.utils.numberformat import format as nformat
from django.utils.safestring import mark_safe, SafeString, SafeUnicode
from django.utils.translation import (ugettext, ugettext_lazy, activate,
    deactivate, gettext_lazy, pgettext, npgettext, to_locale,
    get_language_info, get_language, get_language_from_request)


from .commands.tests import can_run_extraction_tests, can_run_compilation_tests
if can_run_extraction_tests:
    from .commands.extraction import (ExtractorTests, BasicExtractorTests,
        JavascriptExtractorTests, IgnoredExtractorTests, SymlinkExtractorTests,
        CopyPluralFormsExtractorTests, NoWrapExtractorTests,
        NoLocationExtractorTests)
if can_run_compilation_tests:
    from .commands.compilation import (PoFileTests, PoFileContentsTests,
        PercentRenderingTests)
from .contenttypes.tests import ContentTypeTests
from .forms import I18nForm, SelectDateForm, SelectDateWidget, CompanyForm
from .models import Company, TestModel
from .patterns.tests import (URLRedirectWithoutTrailingSlashTests,
    URLTranslationTests, URLDisabledTests, URLTagTests, URLTestCaseBase,
    URLRedirectWithoutTrailingSlashSettingTests, URLNamespaceTests,
    URLPrefixTests, URLResponseTests, URLRedirectTests, PathUnusedTests)
from .test_warnings import DeprecationWarningTests


here = os.path.dirname(os.path.abspath(__file__))

class TranslationTests(TestCase):

    def test_override(self):
        activate('de')
        with translation.override('pl'):
            self.assertEqual(get_language(), 'pl')
        self.assertEqual(get_language(), 'de')
        with translation.override(None):
            self.assertEqual(get_language(), settings.LANGUAGE_CODE)
        self.assertEqual(get_language(), 'de')
        deactivate()

    def test_lazy_objects(self):
        """
        Format string interpolation should work with *_lazy objects.
        """
        s = ugettext_lazy('Add %(name)s')
        d = {'name': 'Ringo'}
        self.assertEqual(u'Add Ringo', s % d)
        with translation.override('de', deactivate=True):
            self.assertEqual(u'Ringo hinzuf\xfcgen', s % d)
            with translation.override('pl'):
                self.assertEqual(u'Dodaj Ringo', s % d)

        # It should be possible to compare *_lazy objects.
        s1 = ugettext_lazy('Add %(name)s')
        self.assertEqual(True, s == s1)
        s2 = gettext_lazy('Add %(name)s')
        s3 = gettext_lazy('Add %(name)s')
        self.assertEqual(True, s2 == s3)
        self.assertEqual(True, s == s2)
        s4 = ugettext_lazy('Some other string')
        self.assertEqual(False, s == s4)

    def test_lazy_pickle(self):
        s1 = ugettext_lazy("test")
        self.assertEqual(unicode(s1), "test")
        s2 = pickle.loads(pickle.dumps(s1))
        self.assertEqual(unicode(s2), "test")

    def test_pgettext(self):
        # Reset translation catalog to include other/locale/de
        extended_locale_paths = settings.LOCALE_PATHS + (
            os.path.join(here, 'other', 'locale'),
        )
        with self.settings(LOCALE_PATHS=extended_locale_paths):
            from django.utils.translation import trans_real
            trans_real._active = local()
            trans_real._translations = {}
            with translation.override('de'):
                self.assertEqual(pgettext("unexisting", "May"), u"May")
                self.assertEqual(pgettext("month name", "May"), u"Mai")
                self.assertEqual(pgettext("verb", "May"), u"Kann")
                self.assertEqual(npgettext("search", "%d result", "%d results", 4) % 4, u"4 Resultate")

    def test_template_tags_pgettext(self):
        """
        Ensure that message contexts are taken into account the {% trans %} and
        {% blocktrans %} template tags.
        Refs #14806.
        """
        # Reset translation catalog to include other/locale/de
        extended_locale_paths = settings.LOCALE_PATHS + (
            os.path.join(here, 'other', 'locale'),
        )
        with self.settings(LOCALE_PATHS=extended_locale_paths):
            from django.utils.translation import trans_real
            trans_real._active = local()
            trans_real._translations = {}
            with translation.override('de'):

                # {% trans %} -----------------------------------

                # Inexisting context...
                t = Template('{% load i18n %}{% trans "May" context "unexisting" %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, 'May')

                # Existing context...
                # Using a literal
                t = Template('{% load i18n %}{% trans "May" context "month name" %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, 'Mai')
                t = Template('{% load i18n %}{% trans "May" context "verb" %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, 'Kann')

                # Using a variable
                t = Template('{% load i18n %}{% trans "May" context message_context %}')
                rendered = t.render(Context({'message_context': 'month name'}))
                self.assertEqual(rendered, 'Mai')
                t = Template('{% load i18n %}{% trans "May" context message_context %}')
                rendered = t.render(Context({'message_context': 'verb'}))
                self.assertEqual(rendered, 'Kann')

                # Using a filter
                t = Template('{% load i18n %}{% trans "May" context message_context|lower %}')
                rendered = t.render(Context({'message_context': 'MONTH NAME'}))
                self.assertEqual(rendered, 'Mai')
                t = Template('{% load i18n %}{% trans "May" context message_context|lower %}')
                rendered = t.render(Context({'message_context': 'VERB'}))
                self.assertEqual(rendered, 'Kann')

                # Using 'as'
                t = Template('{% load i18n %}{% trans "May" context "month name" as var %}Value: {{ var }}')
                rendered = t.render(Context())
                self.assertEqual(rendered, 'Value: Mai')
                t = Template('{% load i18n %}{% trans "May" as var context "verb" %}Value: {{ var }}')
                rendered = t.render(Context())
                self.assertEqual(rendered, 'Value: Kann')

                # Mis-uses
                self.assertRaises(TemplateSyntaxError, Template, '{% load i18n %}{% trans "May" context as var %}{{ var }}')
                self.assertRaises(TemplateSyntaxError, Template, '{% load i18n %}{% trans "May" as var context %}{{ var }}')

                # {% blocktrans %} ------------------------------

                # Inexisting context...
                t = Template('{% load i18n %}{% blocktrans context "unexisting" %}May{% endblocktrans %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, 'May')

                # Existing context...
                # Using a literal
                t = Template('{% load i18n %}{% blocktrans context "month name" %}May{% endblocktrans %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, 'Mai')
                t = Template('{% load i18n %}{% blocktrans context "verb" %}May{% endblocktrans %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, 'Kann')

                # Using a variable
                t = Template('{% load i18n %}{% blocktrans context message_context %}May{% endblocktrans %}')
                rendered = t.render(Context({'message_context': 'month name'}))
                self.assertEqual(rendered, 'Mai')
                t = Template('{% load i18n %}{% blocktrans context message_context %}May{% endblocktrans %}')
                rendered = t.render(Context({'message_context': 'verb'}))
                self.assertEqual(rendered, 'Kann')

                # Using a filter
                t = Template('{% load i18n %}{% blocktrans context message_context|lower %}May{% endblocktrans %}')
                rendered = t.render(Context({'message_context': 'MONTH NAME'}))
                self.assertEqual(rendered, 'Mai')
                t = Template('{% load i18n %}{% blocktrans context message_context|lower %}May{% endblocktrans %}')
                rendered = t.render(Context({'message_context': 'VERB'}))
                self.assertEqual(rendered, 'Kann')

                # Using 'count'
                t = Template('{% load i18n %}{% blocktrans count number=1 context "super search" %}{{ number }} super result{% plural %}{{ number }} super results{% endblocktrans %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, '1 Super-Ergebnis')
                t = Template('{% load i18n %}{% blocktrans count number=2 context "super search" %}{{ number }} super result{% plural %}{{ number }} super results{% endblocktrans %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, '2 Super-Ergebnisse')
                t = Template('{% load i18n %}{% blocktrans context "other super search" count number=1 %}{{ number }} super result{% plural %}{{ number }} super results{% endblocktrans %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, '1 anderen Super-Ergebnis')
                t = Template('{% load i18n %}{% blocktrans context "other super search" count number=2 %}{{ number }} super result{% plural %}{{ number }} super results{% endblocktrans %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, '2 andere Super-Ergebnisse')

                # Using 'with'
                t = Template('{% load i18n %}{% blocktrans with num_comments=5 context "comment count" %}There are {{ num_comments }} comments{% endblocktrans %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, 'Es gibt 5 Kommentare')
                t = Template('{% load i18n %}{% blocktrans with num_comments=5 context "other comment count" %}There are {{ num_comments }} comments{% endblocktrans %}')
                rendered = t.render(Context())
                self.assertEqual(rendered, 'Andere: Es gibt 5 Kommentare')

                # Mis-uses
                self.assertRaises(TemplateSyntaxError, Template, '{% load i18n %}{% blocktrans context with month="May" %}{{ month }}{% endblocktrans %}')
                self.assertRaises(TemplateSyntaxError, Template, '{% load i18n %}{% blocktrans context %}{% endblocktrans %}')
                self.assertRaises(TemplateSyntaxError, Template, '{% load i18n %}{% blocktrans count number=2 context %}{{ number }} super result{% plural %}{{ number }} super results{% endblocktrans %}')


    def test_string_concat(self):
        """
        unicode(string_concat(...)) should not raise a TypeError - #4796
        """
        import django.utils.translation
        self.assertEqual(u'django', unicode(django.utils.translation.string_concat("dja", "ngo")))

    def test_safe_status(self):
        """
        Translating a string requiring no auto-escaping shouldn't change the "safe" status.
        """
        s = mark_safe('Password')
        self.assertEqual(SafeString, type(s))
        with translation.override('de', deactivate=True):
            self.assertEqual(SafeUnicode, type(ugettext(s)))
        self.assertEqual('aPassword', SafeString('a') + s)
        self.assertEqual('Passworda', s + SafeString('a'))
        self.assertEqual('Passworda', s + mark_safe('a'))
        self.assertEqual('aPassword', mark_safe('a') + s)
        self.assertEqual('as', mark_safe('a') + mark_safe('s'))

    def test_maclines(self):
        """
        Translations on files with mac or dos end of lines will be converted
        to unix eof in .po catalogs, and they have to match when retrieved
        """
        from django.utils.translation.trans_real import translation as Trans
        ca_translation = Trans('ca')
        ca_translation._catalog[u'Mac\nEOF\n'] = u'Catalan Mac\nEOF\n'
        ca_translation._catalog[u'Win\nEOF\n'] = u'Catalan Win\nEOF\n'
        with translation.override('ca', deactivate=True):
            self.assertEqual(u'Catalan Mac\nEOF\n', ugettext(u'Mac\rEOF\r'))
            self.assertEqual(u'Catalan Win\nEOF\n', ugettext(u'Win\r\nEOF\r\n'))

    def test_to_locale(self):
        """
        Tests the to_locale function and the special case of Serbian Latin
        (refs #12230 and r11299)
        """
        self.assertEqual(to_locale('en-us'), 'en_US')
        self.assertEqual(to_locale('sr-lat'), 'sr_Lat')

    def test_to_language(self):
        """
        Test the to_language function
        """
        from django.utils.translation.trans_real import to_language
        self.assertEqual(to_language('en_US'), 'en-us')
        self.assertEqual(to_language('sr_Lat'), 'sr-lat')

    @override_settings(LOCALE_PATHS=(os.path.join(here, 'other', 'locale'),))
    def test_bad_placeholder(self):
        """
        Error in translation file should not crash template rendering
        (%(person)s is translated as %(personne)s in fr.po)
        """
        from django.template import Template, Context
        with translation.override('fr'):
            t = Template('{% load i18n %}{% blocktrans %}My name is {{ person }}.{% endblocktrans %}')
            rendered = t.render(Context({'person': 'James'}))
            self.assertEqual(rendered, 'My name is James.')


class FormattingTests(TestCase):

    def setUp(self):
        self.use_i18n = settings.USE_I18N
        self.use_l10n = settings.USE_L10N
        self.use_thousand_separator = settings.USE_THOUSAND_SEPARATOR
        self.thousand_separator = settings.THOUSAND_SEPARATOR
        self.number_grouping = settings.NUMBER_GROUPING
        self.n = decimal.Decimal('66666.666')
        self.f = 99999.999
        self.d = datetime.date(2009, 12, 31)
        self.dt = datetime.datetime(2009, 12, 31, 20, 50)
        self.t = datetime.time(10, 15, 48)
        self.l = 10000L
        self.ctxt = Context({
            'n': self.n,
            't': self.t,
            'd': self.d,
            'dt': self.dt,
            'f': self.f,
            'l': self.l,
        })

    def tearDown(self):
        # Restore defaults
        settings.USE_I18N = self.use_i18n
        settings.USE_L10N = self.use_l10n
        settings.USE_THOUSAND_SEPARATOR = self.use_thousand_separator
        settings.THOUSAND_SEPARATOR = self.thousand_separator
        settings.NUMBER_GROUPING = self.number_grouping

    def test_locale_independent(self):
        """
        Localization of numbers
        """
        with self.settings(USE_L10N=True, USE_THOUSAND_SEPARATOR=False):
            self.assertEqual(u'66666.66', nformat(self.n, decimal_sep='.', decimal_pos=2, grouping=3, thousand_sep=','))
            self.assertEqual(u'66666A6', nformat(self.n, decimal_sep='A', decimal_pos=1, grouping=1, thousand_sep='B'))
            self.assertEqual(u'66666', nformat(self.n, decimal_sep='X', decimal_pos=0, grouping=1, thousand_sep='Y'))

        with self.settings(USE_L10N=True, USE_THOUSAND_SEPARATOR=True):
            self.assertEqual(u'66,666.66', nformat(self.n, decimal_sep='.', decimal_pos=2, grouping=3, thousand_sep=','))
            self.assertEqual(u'6B6B6B6B6A6', nformat(self.n, decimal_sep='A', decimal_pos=1, grouping=1, thousand_sep='B'))
            self.assertEqual(u'-66666.6', nformat(-66666.666, decimal_sep='.', decimal_pos=1))
            self.assertEqual(u'-66666.0', nformat(int('-66666'), decimal_sep='.', decimal_pos=1))
            self.assertEqual(u'10000.0', nformat(self.l, decimal_sep='.', decimal_pos=1))
            # This unusual grouping/force_grouping combination may be triggered by the intcomma filter (#17414)
            self.assertEqual(u'10000', nformat(self.l, decimal_sep='.', decimal_pos=0, grouping=0, force_grouping=True))

            # date filter
            self.assertEqual(u'31.12.2009 в 20:50', Template('{{ dt|date:"d.m.Y в H:i" }}').render(self.ctxt))
            self.assertEqual(u'⌚ 10:15', Template('{{ t|time:"⌚ H:i" }}').render(self.ctxt))

    def test_l10n_disabled(self):
        """
        Catalan locale with format i18n disabled translations will be used,
        but not formats
        """
        settings.USE_L10N = False
        with translation.override('ca', deactivate=True):
            self.assertEqual(u'N j, Y', get_format('DATE_FORMAT'))
            self.assertEqual(0, get_format('FIRST_DAY_OF_WEEK'))
            self.assertEqual(u'.', get_format('DECIMAL_SEPARATOR'))
            self.assertEqual(u'10:15 a.m.', time_format(self.t))
            self.assertEqual(u'des. 31, 2009', date_format(self.d))
            self.assertEqual(u'desembre 2009', date_format(self.d, 'YEAR_MONTH_FORMAT'))
            self.assertEqual(u'12/31/2009 8:50 p.m.', date_format(self.dt, 'SHORT_DATETIME_FORMAT'))
            self.assertEqual(u'No localizable', localize('No localizable'))
            self.assertEqual(u'66666.666', localize(self.n))
            self.assertEqual(u'99999.999', localize(self.f))
            self.assertEqual(u'10000', localize(self.l))
            self.assertEqual(u'des. 31, 2009', localize(self.d))
            self.assertEqual(u'des. 31, 2009, 8:50 p.m.', localize(self.dt))
            self.assertEqual(u'66666.666', Template('{{ n }}').render(self.ctxt))
            self.assertEqual(u'99999.999', Template('{{ f }}').render(self.ctxt))
            self.assertEqual(u'des. 31, 2009', Template('{{ d }}').render(self.ctxt))
            self.assertEqual(u'des. 31, 2009, 8:50 p.m.', Template('{{ dt }}').render(self.ctxt))
            self.assertEqual(u'66666.67', Template('{{ n|floatformat:2 }}').render(self.ctxt))
            self.assertEqual(u'100000.0', Template('{{ f|floatformat }}').render(self.ctxt))
            self.assertEqual(u'10:15 a.m.', Template('{{ t|time:"TIME_FORMAT" }}').render(self.ctxt))
            self.assertEqual(u'12/31/2009', Template('{{ d|date:"SHORT_DATE_FORMAT" }}').render(self.ctxt))
            self.assertEqual(u'12/31/2009 8:50 p.m.', Template('{{ dt|date:"SHORT_DATETIME_FORMAT" }}').render(self.ctxt))

            form = I18nForm({
                'decimal_field': u'66666,666',
                'float_field': u'99999,999',
                'date_field': u'31/12/2009',
                'datetime_field': u'31/12/2009 20:50',
                'time_field': u'20:50',
                'integer_field': u'1.234',
            })
            self.assertEqual(False, form.is_valid())
            self.assertEqual([u'Introdu\xefu un n\xfamero.'], form.errors['float_field'])
            self.assertEqual([u'Introdu\xefu un n\xfamero.'], form.errors['decimal_field'])
            self.assertEqual([u'Introdu\xefu una data v\xe0lida.'], form.errors['date_field'])
            self.assertEqual([u'Introdu\xefu una data/hora v\xe0lides.'], form.errors['datetime_field'])
            self.assertEqual([u'Introdu\xefu un n\xfamero sencer.'], form.errors['integer_field'])

            form2 = SelectDateForm({
                'date_field_month': u'12',
                'date_field_day': u'31',
                'date_field_year': u'2009'
            })
            self.assertEqual(True, form2.is_valid())
            self.assertEqual(datetime.date(2009, 12, 31), form2.cleaned_data['date_field'])
            self.assertHTMLEqual(
                u'<select name="mydate_month" id="id_mydate_month">\n<option value="1">gener</option>\n<option value="2">febrer</option>\n<option value="3">mar\xe7</option>\n<option value="4">abril</option>\n<option value="5">maig</option>\n<option value="6">juny</option>\n<option value="7">juliol</option>\n<option value="8">agost</option>\n<option value="9">setembre</option>\n<option value="10">octubre</option>\n<option value="11">novembre</option>\n<option value="12" selected="selected">desembre</option>\n</select>\n<select name="mydate_day" id="id_mydate_day">\n<option value="1">1</option>\n<option value="2">2</option>\n<option value="3">3</option>\n<option value="4">4</option>\n<option value="5">5</option>\n<option value="6">6</option>\n<option value="7">7</option>\n<option value="8">8</option>\n<option value="9">9</option>\n<option value="10">10</option>\n<option value="11">11</option>\n<option value="12">12</option>\n<option value="13">13</option>\n<option value="14">14</option>\n<option value="15">15</option>\n<option value="16">16</option>\n<option value="17">17</option>\n<option value="18">18</option>\n<option value="19">19</option>\n<option value="20">20</option>\n<option value="21">21</option>\n<option value="22">22</option>\n<option value="23">23</option>\n<option value="24">24</option>\n<option value="25">25</option>\n<option value="26">26</option>\n<option value="27">27</option>\n<option value="28">28</option>\n<option value="29">29</option>\n<option value="30">30</option>\n<option value="31" selected="selected">31</option>\n</select>\n<select name="mydate_year" id="id_mydate_year">\n<option value="2009" selected="selected">2009</option>\n<option value="2010">2010</option>\n<option value="2011">2011</option>\n<option value="2012">2012</option>\n<option value="2013">2013</option>\n<option value="2014">2014</option>\n<option value="2015">2015</option>\n<option value="2016">2016</option>\n<option value="2017">2017</option>\n<option value="2018">2018</option>\n</select>',
                SelectDateWidget(years=range(2009, 2019)).render('mydate', datetime.date(2009, 12, 31))
            )

            # We shouldn't change the behavior of the floatformat filter re:
            # thousand separator and grouping when USE_L10N is False even
            # if the USE_THOUSAND_SEPARATOR, NUMBER_GROUPING and
            # THOUSAND_SEPARATOR settings are specified
            with self.settings(USE_THOUSAND_SEPARATOR=True,
                    NUMBER_GROUPING=1, THOUSAND_SEPARATOR='!'):
                self.assertEqual(u'66666.67', Template('{{ n|floatformat:2 }}').render(self.ctxt))
                self.assertEqual(u'100000.0', Template('{{ f|floatformat }}').render(self.ctxt))

    def test_false_like_locale_formats(self):
        """
        Ensure that the active locale's formats take precedence over the
        default settings even if they would be interpreted as False in a
        conditional test (e.g. 0 or empty string).
        Refs #16938.
        """
        from django.conf.locale.fr import formats as fr_formats

        # Back up original formats
        backup_THOUSAND_SEPARATOR = fr_formats.THOUSAND_SEPARATOR
        backup_FIRST_DAY_OF_WEEK = fr_formats.FIRST_DAY_OF_WEEK

        # Set formats that would get interpreted as False in a conditional test
        fr_formats.THOUSAND_SEPARATOR = ''
        fr_formats.FIRST_DAY_OF_WEEK = 0

        with translation.override('fr'):
            with self.settings(USE_L10N=True, USE_THOUSAND_SEPARATOR=True,
                               THOUSAND_SEPARATOR='!'):
                self.assertEqual('', get_format('THOUSAND_SEPARATOR'))
                # Even a second time (after the format has been cached)...
                self.assertEqual('', get_format('THOUSAND_SEPARATOR'))

            with self.settings(USE_L10N=True, FIRST_DAY_OF_WEEK=1):
                self.assertEqual(0, get_format('FIRST_DAY_OF_WEEK'))
                # Even a second time (after the format has been cached)...
                self.assertEqual(0, get_format('FIRST_DAY_OF_WEEK'))

        # Restore original formats
        fr_formats.THOUSAND_SEPARATOR = backup_THOUSAND_SEPARATOR
        fr_formats.FIRST_DAY_OF_WEEK = backup_FIRST_DAY_OF_WEEK

    def test_l10n_enabled(self):
        settings.USE_L10N = True
        # Catalan locale
        with translation.override('ca', deactivate=True):
            self.assertEqual('j \d\e F \d\e Y', get_format('DATE_FORMAT'))
            self.assertEqual(1, get_format('FIRST_DAY_OF_WEEK'))
            self.assertEqual(',', get_format('DECIMAL_SEPARATOR'))
            self.assertEqual(u'10:15:48', time_format(self.t))
            self.assertEqual(u'31 de desembre de 2009', date_format(self.d))
            self.assertEqual(u'desembre del 2009', date_format(self.d, 'YEAR_MONTH_FORMAT'))
            self.assertEqual(u'31/12/2009 20:50', date_format(self.dt, 'SHORT_DATETIME_FORMAT'))
            self.assertEqual('No localizable', localize('No localizable'))

            settings.USE_THOUSAND_SEPARATOR = True
            self.assertEqual(u'66.666,666', localize(self.n))
            self.assertEqual(u'99.999,999', localize(self.f))
            self.assertEqual(u'10.000', localize(self.l))
            self.assertEqual(u'True', localize(True))

            settings.USE_THOUSAND_SEPARATOR = False
            self.assertEqual(u'66666,666', localize(self.n))
            self.assertEqual(u'99999,999', localize(self.f))
            self.assertEqual(u'10000', localize(self.l))
            self.assertEqual(u'31 de desembre de 2009', localize(self.d))
            self.assertEqual(u'31 de desembre de 2009 a les 20:50', localize(self.dt))

            settings.USE_THOUSAND_SEPARATOR = True
            self.assertEqual(u'66.666,666', Template('{{ n }}').render(self.ctxt))
            self.assertEqual(u'99.999,999', Template('{{ f }}').render(self.ctxt))
            self.assertEqual(u'10.000', Template('{{ l }}').render(self.ctxt))

            form3 = I18nForm({
                'decimal_field': u'66.666,666',
                'float_field': u'99.999,999',
                'date_field': u'31/12/2009',
                'datetime_field': u'31/12/2009 20:50',
                'time_field': u'20:50',
                'integer_field': u'1.234',
            })
            self.assertEqual(True, form3.is_valid())
            self.assertEqual(decimal.Decimal('66666.666'), form3.cleaned_data['decimal_field'])
            self.assertEqual(99999.999, form3.cleaned_data['float_field'])
            self.assertEqual(datetime.date(2009, 12, 31), form3.cleaned_data['date_field'])
            self.assertEqual(datetime.datetime(2009, 12, 31, 20, 50), form3.cleaned_data['datetime_field'])
            self.assertEqual(datetime.time(20, 50), form3.cleaned_data['time_field'])
            self.assertEqual(1234, form3.cleaned_data['integer_field'])

            settings.USE_THOUSAND_SEPARATOR = False
            self.assertEqual(u'66666,666', Template('{{ n }}').render(self.ctxt))
            self.assertEqual(u'99999,999', Template('{{ f }}').render(self.ctxt))
            self.assertEqual(u'31 de desembre de 2009', Template('{{ d }}').render(self.ctxt))
            self.assertEqual(u'31 de desembre de 2009 a les 20:50', Template('{{ dt }}').render(self.ctxt))
            self.assertEqual(u'66666,67', Template('{{ n|floatformat:2 }}').render(self.ctxt))
            self.assertEqual(u'100000,0', Template('{{ f|floatformat }}').render(self.ctxt))
            self.assertEqual(u'10:15:48', Template('{{ t|time:"TIME_FORMAT" }}').render(self.ctxt))
            self.assertEqual(u'31/12/2009', Template('{{ d|date:"SHORT_DATE_FORMAT" }}').render(self.ctxt))
            self.assertEqual(u'31/12/2009 20:50', Template('{{ dt|date:"SHORT_DATETIME_FORMAT" }}').render(self.ctxt))
            self.assertEqual(date_format(datetime.datetime.now(), "DATE_FORMAT"), Template('{% now "DATE_FORMAT" %}').render(self.ctxt))

            form4 = I18nForm({
                'decimal_field': u'66666,666',
                'float_field': u'99999,999',
                'date_field': u'31/12/2009',
                'datetime_field': u'31/12/2009 20:50',
                'time_field': u'20:50',
                'integer_field': u'1234',
            })
            self.assertEqual(True, form4.is_valid())
            self.assertEqual(decimal.Decimal('66666.666'), form4.cleaned_data['decimal_field'])
            self.assertEqual(99999.999, form4.cleaned_data['float_field'])
            self.assertEqual(datetime.date(2009, 12, 31), form4.cleaned_data['date_field'])
            self.assertEqual(datetime.datetime(2009, 12, 31, 20, 50), form4.cleaned_data['datetime_field'])
            self.assertEqual(datetime.time(20, 50), form4.cleaned_data['time_field'])
            self.assertEqual(1234, form4.cleaned_data['integer_field'])

            form5 = SelectDateForm({
                'date_field_month': u'12',
                'date_field_day': u'31',
                'date_field_year': u'2009'
            })
            self.assertEqual(True, form5.is_valid())
            self.assertEqual(datetime.date(2009, 12, 31), form5.cleaned_data['date_field'])
            self.assertHTMLEqual(
                u'<select name="mydate_day" id="id_mydate_day">\n<option value="1">1</option>\n<option value="2">2</option>\n<option value="3">3</option>\n<option value="4">4</option>\n<option value="5">5</option>\n<option value="6">6</option>\n<option value="7">7</option>\n<option value="8">8</option>\n<option value="9">9</option>\n<option value="10">10</option>\n<option value="11">11</option>\n<option value="12">12</option>\n<option value="13">13</option>\n<option value="14">14</option>\n<option value="15">15</option>\n<option value="16">16</option>\n<option value="17">17</option>\n<option value="18">18</option>\n<option value="19">19</option>\n<option value="20">20</option>\n<option value="21">21</option>\n<option value="22">22</option>\n<option value="23">23</option>\n<option value="24">24</option>\n<option value="25">25</option>\n<option value="26">26</option>\n<option value="27">27</option>\n<option value="28">28</option>\n<option value="29">29</option>\n<option value="30">30</option>\n<option value="31" selected="selected">31</option>\n</select>\n<select name="mydate_month" id="id_mydate_month">\n<option value="1">gener</option>\n<option value="2">febrer</option>\n<option value="3">mar\xe7</option>\n<option value="4">abril</option>\n<option value="5">maig</option>\n<option value="6">juny</option>\n<option value="7">juliol</option>\n<option value="8">agost</option>\n<option value="9">setembre</option>\n<option value="10">octubre</option>\n<option value="11">novembre</option>\n<option value="12" selected="selected">desembre</option>\n</select>\n<select name="mydate_year" id="id_mydate_year">\n<option value="2009" selected="selected">2009</option>\n<option value="2010">2010</option>\n<option value="2011">2011</option>\n<option value="2012">2012</option>\n<option value="2013">2013</option>\n<option value="2014">2014</option>\n<option value="2015">2015</option>\n<option value="2016">2016</option>\n<option value="2017">2017</option>\n<option value="2018">2018</option>\n</select>',
                SelectDateWidget(years=range(2009, 2019)).render('mydate', datetime.date(2009, 12, 31))
            )

        # Russian locale (with E as month)
        with translation.override('ru', deactivate=True):
            self.assertHTMLEqual(
                    u'<select name="mydate_day" id="id_mydate_day">\n<option value="1">1</option>\n<option value="2">2</option>\n<option value="3">3</option>\n<option value="4">4</option>\n<option value="5">5</option>\n<option value="6">6</option>\n<option value="7">7</option>\n<option value="8">8</option>\n<option value="9">9</option>\n<option value="10">10</option>\n<option value="11">11</option>\n<option value="12">12</option>\n<option value="13">13</option>\n<option value="14">14</option>\n<option value="15">15</option>\n<option value="16">16</option>\n<option value="17">17</option>\n<option value="18">18</option>\n<option value="19">19</option>\n<option value="20">20</option>\n<option value="21">21</option>\n<option value="22">22</option>\n<option value="23">23</option>\n<option value="24">24</option>\n<option value="25">25</option>\n<option value="26">26</option>\n<option value="27">27</option>\n<option value="28">28</option>\n<option value="29">29</option>\n<option value="30">30</option>\n<option value="31" selected="selected">31</option>\n</select>\n<select name="mydate_month" id="id_mydate_month">\n<option value="1">\u042f\u043d\u0432\u0430\u0440\u044c</option>\n<option value="2">\u0424\u0435\u0432\u0440\u0430\u043b\u044c</option>\n<option value="3">\u041c\u0430\u0440\u0442</option>\n<option value="4">\u0410\u043f\u0440\u0435\u043b\u044c</option>\n<option value="5">\u041c\u0430\u0439</option>\n<option value="6">\u0418\u044e\u043d\u044c</option>\n<option value="7">\u0418\u044e\u043b\u044c</option>\n<option value="8">\u0410\u0432\u0433\u0443\u0441\u0442</option>\n<option value="9">\u0421\u0435\u043d\u0442\u044f\u0431\u0440\u044c</option>\n<option value="10">\u041e\u043a\u0442\u044f\u0431\u0440\u044c</option>\n<option value="11">\u041d\u043e\u044f\u0431\u0440\u044c</option>\n<option value="12" selected="selected">\u0414\u0435\u043a\u0430\u0431\u0440\u044c</option>\n</select>\n<select name="mydate_year" id="id_mydate_year">\n<option value="2009" selected="selected">2009</option>\n<option value="2010">2010</option>\n<option value="2011">2011</option>\n<option value="2012">2012</option>\n<option value="2013">2013</option>\n<option value="2014">2014</option>\n<option value="2015">2015</option>\n<option value="2016">2016</option>\n<option value="2017">2017</option>\n<option value="2018">2018</option>\n</select>',
                    SelectDateWidget(years=range(2009, 2019)).render('mydate', datetime.date(2009, 12, 31))
            )

        # English locale
        with translation.override('en', deactivate=True):
            self.assertEqual('N j, Y', get_format('DATE_FORMAT'))
            self.assertEqual(0, get_format('FIRST_DAY_OF_WEEK'))
            self.assertEqual('.', get_format('DECIMAL_SEPARATOR'))
            self.assertEqual(u'Dec. 31, 2009', date_format(self.d))
            self.assertEqual(u'December 2009', date_format(self.d, 'YEAR_MONTH_FORMAT'))
            self.assertEqual(u'12/31/2009 8:50 p.m.', date_format(self.dt, 'SHORT_DATETIME_FORMAT'))
            self.assertEqual(u'No localizable', localize('No localizable'))

            settings.USE_THOUSAND_SEPARATOR = True
            self.assertEqual(u'66,666.666', localize(self.n))
            self.assertEqual(u'99,999.999', localize(self.f))
            self.assertEqual(u'10,000', localize(self.l))

            settings.USE_THOUSAND_SEPARATOR = False
            self.assertEqual(u'66666.666', localize(self.n))
            self.assertEqual(u'99999.999', localize(self.f))
            self.assertEqual(u'10000', localize(self.l))
            self.assertEqual(u'Dec. 31, 2009', localize(self.d))
            self.assertEqual(u'Dec. 31, 2009, 8:50 p.m.', localize(self.dt))

            settings.USE_THOUSAND_SEPARATOR = True
            self.assertEqual(u'66,666.666', Template('{{ n }}').render(self.ctxt))
            self.assertEqual(u'99,999.999', Template('{{ f }}').render(self.ctxt))
            self.assertEqual(u'10,000', Template('{{ l }}').render(self.ctxt))

            settings.USE_THOUSAND_SEPARATOR = False
            self.assertEqual(u'66666.666', Template('{{ n }}').render(self.ctxt))
            self.assertEqual(u'99999.999', Template('{{ f }}').render(self.ctxt))
            self.assertEqual(u'Dec. 31, 2009', Template('{{ d }}').render(self.ctxt))
            self.assertEqual(u'Dec. 31, 2009, 8:50 p.m.', Template('{{ dt }}').render(self.ctxt))
            self.assertEqual(u'66666.67', Template('{{ n|floatformat:2 }}').render(self.ctxt))
            self.assertEqual(u'100000.0', Template('{{ f|floatformat }}').render(self.ctxt))
            self.assertEqual(u'12/31/2009', Template('{{ d|date:"SHORT_DATE_FORMAT" }}').render(self.ctxt))
            self.assertEqual(u'12/31/2009 8:50 p.m.', Template('{{ dt|date:"SHORT_DATETIME_FORMAT" }}').render(self.ctxt))

            form5 = I18nForm({
                'decimal_field': u'66666.666',
                'float_field': u'99999.999',
                'date_field': u'12/31/2009',
                'datetime_field': u'12/31/2009 20:50',
                'time_field': u'20:50',
                'integer_field': u'1234',
            })
            self.assertEqual(True, form5.is_valid())
            self.assertEqual(decimal.Decimal('66666.666'), form5.cleaned_data['decimal_field'])
            self.assertEqual(99999.999, form5.cleaned_data['float_field'])
            self.assertEqual(datetime.date(2009, 12, 31), form5.cleaned_data['date_field'])
            self.assertEqual(datetime.datetime(2009, 12, 31, 20, 50), form5.cleaned_data['datetime_field'])
            self.assertEqual(datetime.time(20, 50), form5.cleaned_data['time_field'])
            self.assertEqual(1234, form5.cleaned_data['integer_field'])

            form6 = SelectDateForm({
                'date_field_month': u'12',
                'date_field_day': u'31',
                'date_field_year': u'2009'
            })
            self.assertEqual(True, form6.is_valid())
            self.assertEqual(datetime.date(2009, 12, 31), form6.cleaned_data['date_field'])
            self.assertHTMLEqual(
                u'<select name="mydate_month" id="id_mydate_month">\n<option value="1">January</option>\n<option value="2">February</option>\n<option value="3">March</option>\n<option value="4">April</option>\n<option value="5">May</option>\n<option value="6">June</option>\n<option value="7">July</option>\n<option value="8">August</option>\n<option value="9">September</option>\n<option value="10">October</option>\n<option value="11">November</option>\n<option value="12" selected="selected">December</option>\n</select>\n<select name="mydate_day" id="id_mydate_day">\n<option value="1">1</option>\n<option value="2">2</option>\n<option value="3">3</option>\n<option value="4">4</option>\n<option value="5">5</option>\n<option value="6">6</option>\n<option value="7">7</option>\n<option value="8">8</option>\n<option value="9">9</option>\n<option value="10">10</option>\n<option value="11">11</option>\n<option value="12">12</option>\n<option value="13">13</option>\n<option value="14">14</option>\n<option value="15">15</option>\n<option value="16">16</option>\n<option value="17">17</option>\n<option value="18">18</option>\n<option value="19">19</option>\n<option value="20">20</option>\n<option value="21">21</option>\n<option value="22">22</option>\n<option value="23">23</option>\n<option value="24">24</option>\n<option value="25">25</option>\n<option value="26">26</option>\n<option value="27">27</option>\n<option value="28">28</option>\n<option value="29">29</option>\n<option value="30">30</option>\n<option value="31" selected="selected">31</option>\n</select>\n<select name="mydate_year" id="id_mydate_year">\n<option value="2009" selected="selected">2009</option>\n<option value="2010">2010</option>\n<option value="2011">2011</option>\n<option value="2012">2012</option>\n<option value="2013">2013</option>\n<option value="2014">2014</option>\n<option value="2015">2015</option>\n<option value="2016">2016</option>\n<option value="2017">2017</option>\n<option value="2018">2018</option>\n</select>',
                SelectDateWidget(years=range(2009, 2019)).render('mydate', datetime.date(2009, 12, 31))
            )

    def test_sub_locales(self):
        """
        Check if sublocales fall back to the main locale
        """
        with self.settings(USE_L10N=True, USE_THOUSAND_SEPARATOR=True):
            with translation.override('de-at', deactivate=True):
                self.assertEqual(u'66.666,666', Template('{{ n }}').render(self.ctxt))
            with translation.override('es-us', deactivate=True):
                self.assertEqual(u'31 de diciembre de 2009', date_format(self.d))

    def test_localized_input(self):
        """
        Tests if form input is correctly localized
        """
        settings.USE_L10N = True
        with translation.override('de-at', deactivate=True):
            form6 = CompanyForm({
                'name': u'acme',
                'date_added': datetime.datetime(2009, 12, 31, 6, 0, 0),
                'cents_paid': decimal.Decimal('59.47'),
                'products_delivered': 12000,
            })
            self.assertEqual(True, form6.is_valid())
            self.assertHTMLEqual(
                form6.as_ul(),
                u'<li><label for="id_name">Name:</label> <input id="id_name" type="text" name="name" value="acme" maxlength="50" /></li>\n<li><label for="id_date_added">Date added:</label> <input type="text" name="date_added" value="31.12.2009 06:00:00" id="id_date_added" /></li>\n<li><label for="id_cents_paid">Cents paid:</label> <input type="text" name="cents_paid" value="59,47" id="id_cents_paid" /></li>\n<li><label for="id_products_delivered">Products delivered:</label> <input type="text" name="products_delivered" value="12000" id="id_products_delivered" /></li>'
            )
            self.assertEqual(localize_input(datetime.datetime(2009, 12, 31, 6, 0, 0)), '31.12.2009 06:00:00')
            self.assertEqual(datetime.datetime(2009, 12, 31, 6, 0, 0), form6.cleaned_data['date_added'])
            with self.settings(USE_THOUSAND_SEPARATOR=True):
                # Checking for the localized "products_delivered" field
                self.assertTrue(u'<input type="text" name="products_delivered" value="12.000" id="id_products_delivered" />' in form6.as_ul())

    def test_iter_format_modules(self):
        """
        Tests the iter_format_modules function.
        """
        settings.USE_L10N = True
        with translation.override('de-at', deactivate=True):
            de_format_mod = import_module('django.conf.locale.de.formats')
            self.assertEqual(list(iter_format_modules('de')), [de_format_mod])
            with self.settings(FORMAT_MODULE_PATH='regressiontests.i18n.other.locale'):
                test_de_format_mod = import_module('regressiontests.i18n.other.locale.de.formats')
                self.assertEqual(list(iter_format_modules('de')), [test_de_format_mod, de_format_mod])

    def test_iter_format_modules_stability(self):
        """
        Tests the iter_format_modules function always yields format modules in
        a stable and correct order in presence of both base ll and ll_CC formats.
        """
        settings.USE_L10N = True
        en_format_mod = import_module('django.conf.locale.en.formats')
        en_gb_format_mod = import_module('django.conf.locale.en_GB.formats')
        self.assertEqual(list(iter_format_modules('en-gb')), [en_gb_format_mod, en_format_mod])

    def test_get_format_modules_lang(self):
        with self.settings(USE_L10N=True):
            with translation.override('de', deactivate=True):
                self.assertEqual('.', get_format('DECIMAL_SEPARATOR', lang='en'))

    def test_get_format_modules_stability(self):
        with self.settings(USE_L10N=True,
                FORMAT_MODULE_PATH='regressiontests.i18n.other.locale'):
            with translation.override('de', deactivate=True):
                old = "%r" % get_format_modules(reverse=True)
                new = "%r" % get_format_modules(reverse=True) # second try
                self.assertEqual(new, old, 'Value returned by get_formats_modules() must be preserved between calls.')

    def test_localize_templatetag_and_filter(self):
        """
        Tests the {% localize %} templatetag
        """
        context = Context({'value': 3.14 })
        template1 = Template("{% load l10n %}{% localize %}{{ value }}{% endlocalize %};{% localize on %}{{ value }}{% endlocalize %}")
        template2 = Template("{% load l10n %}{{ value }};{% localize off %}{{ value }};{% endlocalize %}{{ value }}")
        template3 = Template('{% load l10n %}{{ value }};{{ value|unlocalize }}')
        template4 = Template('{% load l10n %}{{ value }};{{ value|localize }}')
        output1 = '3,14;3,14'
        output2 = '3,14;3.14;3,14'
        output3 = '3,14;3.14'
        output4 = '3.14;3,14'
        with translation.override('de', deactivate=True):
            with self.settings(USE_L10N=False):
                self.assertEqual(template1.render(context), output1)
                self.assertEqual(template4.render(context), output4)
            with self.settings(USE_L10N=True):
                self.assertEqual(template1.render(context), output1)
                self.assertEqual(template2.render(context), output2)
                self.assertEqual(template3.render(context), output3)

class MiscTests(TestCase):

    def setUp(self):
        self.rf = RequestFactory()

    def test_parse_spec_http_header(self):
        """
        Testing HTTP header parsing. First, we test that we can parse the
        values according to the spec (and that we extract all the pieces in
        the right order).
        """
        from django.utils.translation.trans_real import parse_accept_lang_header
        p = parse_accept_lang_header
        # Good headers.
        self.assertEqual([('de', 1.0)], p('de'))
        self.assertEqual([('en-AU', 1.0)], p('en-AU'))
        self.assertEqual([('*', 1.0)], p('*;q=1.00'))
        self.assertEqual([('en-AU', 0.123)], p('en-AU;q=0.123'))
        self.assertEqual([('en-au', 0.5)], p('en-au;q=0.5'))
        self.assertEqual([('en-au', 1.0)], p('en-au;q=1.0'))
        self.assertEqual([('da', 1.0), ('en', 0.5), ('en-gb', 0.25)], p('da, en-gb;q=0.25, en;q=0.5'))
        self.assertEqual([('en-au-xx', 1.0)], p('en-au-xx'))
        self.assertEqual([('de', 1.0), ('en-au', 0.75), ('en-us', 0.5), ('en', 0.25), ('es', 0.125), ('fa', 0.125)], p('de,en-au;q=0.75,en-us;q=0.5,en;q=0.25,es;q=0.125,fa;q=0.125'))
        self.assertEqual([('*', 1.0)], p('*'))
        self.assertEqual([('de', 1.0)], p('de;q=0.'))
        self.assertEqual([('en', 1.0), ('*', 0.5)], p('en; q=1.0, * ; q=0.5'))
        self.assertEqual([], p(''))

        # Bad headers; should always return [].
        self.assertEqual([], p('en-gb;q=1.0000'))
        self.assertEqual([], p('en;q=0.1234'))
        self.assertEqual([], p('en;q=.2'))
        self.assertEqual([], p('abcdefghi-au'))
        self.assertEqual([], p('**'))
        self.assertEqual([], p('en,,gb'))
        self.assertEqual([], p('en-au;q=0.1.0'))
        self.assertEqual([], p('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXZ,en'))
        self.assertEqual([], p('da, en-gb;q=0.8, en;q=0.7,#'))
        self.assertEqual([], p('de;q=2.0'))
        self.assertEqual([], p('de;q=0.a'))
        self.assertEqual([], p(''))

    def test_parse_literal_http_header(self):
        """
        Now test that we parse a literal HTTP header correctly.
        """
        g = get_language_from_request
        r = self.rf.get('/')
        r.COOKIES = {}
        r.META = {'HTTP_ACCEPT_LANGUAGE': 'pt-br'}
        self.assertEqual('pt-br', g(r))

        r.META = {'HTTP_ACCEPT_LANGUAGE': 'pt'}
        self.assertEqual('pt', g(r))

        r.META = {'HTTP_ACCEPT_LANGUAGE': 'es,de'}
        self.assertEqual('es', g(r))

        r.META = {'HTTP_ACCEPT_LANGUAGE': 'es-ar,de'}
        self.assertEqual('es-ar', g(r))

        # This test assumes there won't be a Django translation to a US
        # variation of the Spanish language, a safe assumption. When the
        # user sets it as the preferred language, the main 'es'
        # translation should be selected instead.
        r.META = {'HTTP_ACCEPT_LANGUAGE': 'es-us'}
        self.assertEqual(g(r), 'es')

        # This tests the following scenario: there isn't a main language (zh)
        # translation of Django but there is a translation to variation (zh_CN)
        # the user sets zh-cn as the preferred language, it should be selected
        # by Django without falling back nor ignoring it.
        r.META = {'HTTP_ACCEPT_LANGUAGE': 'zh-cn,de'}
        self.assertEqual(g(r), 'zh-cn')

    def test_parse_language_cookie(self):
        """
        Now test that we parse language preferences stored in a cookie correctly.
        """
        g = get_language_from_request
        r = self.rf.get('/')
        r.COOKIES = {settings.LANGUAGE_COOKIE_NAME: 'pt-br'}
        r.META = {}
        self.assertEqual('pt-br', g(r))

        r.COOKIES = {settings.LANGUAGE_COOKIE_NAME: 'pt'}
        r.META = {}
        self.assertEqual('pt', g(r))

        r.COOKIES = {settings.LANGUAGE_COOKIE_NAME: 'es'}
        r.META = {'HTTP_ACCEPT_LANGUAGE': 'de'}
        self.assertEqual('es', g(r))

        # This test assumes there won't be a Django translation to a US
        # variation of the Spanish language, a safe assumption. When the
        # user sets it as the preferred language, the main 'es'
        # translation should be selected instead.
        r.COOKIES = {settings.LANGUAGE_COOKIE_NAME: 'es-us'}
        r.META = {}
        self.assertEqual(g(r), 'es')

        # This tests the following scenario: there isn't a main language (zh)
        # translation of Django but there is a translation to variation (zh_CN)
        # the user sets zh-cn as the preferred language, it should be selected
        # by Django without falling back nor ignoring it.
        r.COOKIES = {settings.LANGUAGE_COOKIE_NAME: 'zh-cn'}
        r.META = {'HTTP_ACCEPT_LANGUAGE': 'de'}
        self.assertEqual(g(r), 'zh-cn')

    def test_get_language_from_path(self):
        from django.utils.translation.trans_real import get_language_from_path as g
        self.assertEqual(g('/pl/'), 'pl')
        self.assertEqual(g('/pl'), 'pl')
        self.assertEqual(g('/xyz/'), None)

    def test_get_language_from_path(self):
        from django.utils.translation.trans_null import get_language_from_path as g
        self.assertEqual(g('/pl/'), None)
        self.assertEqual(g('/pl'), None)
        self.assertEqual(g('/xyz/'), None)

    def test_percent_in_translatable_block(self):
        extended_locale_paths = settings.LOCALE_PATHS + (
            os.path.join(here, 'other', 'locale'),
        )
        with self.settings(LOCALE_PATHS=extended_locale_paths):
            t_sing = Template("{% load i18n %}{% blocktrans %}The result was {{ percent }}%{% endblocktrans %}")
            t_plur = Template("{% load i18n %}{% blocktrans count num as number %}{{ percent }}% represents {{ num }} object{% plural %}{{ percent }}% represents {{ num }} objects{% endblocktrans %}")
            with translation.override('de'):
                self.assertEqual(t_sing.render(Context({'percent': 42})), u'Das Ergebnis war 42%')
                self.assertEqual(t_plur.render(Context({'percent': 42, 'num': 1})), u'42% stellt 1 Objekt dar')
                self.assertEqual(t_plur.render(Context({'percent': 42, 'num': 4})), u'42% stellt 4 Objekte dar')


class ResolutionOrderI18NTests(TestCase):

    def setUp(self):
        from django.utils.translation import trans_real
        # Okay, this is brutal, but we have no other choice to fully reset
        # the translation framework
        trans_real._active = local()
        trans_real._translations = {}
        activate('de')

    def tearDown(self):
        deactivate()

    def assertUgettext(self, msgid, msgstr):
        result = ugettext(msgid)
        self.assertTrue(msgstr in result, ("The string '%s' isn't in the "
            "translation of '%s'; the actual result is '%s'." % (msgstr, msgid, result)))

class AppResolutionOrderI18NTests(ResolutionOrderI18NTests):

    def setUp(self):
        self.old_installed_apps = settings.INSTALLED_APPS
        settings.INSTALLED_APPS = ['regressiontests.i18n.resolution'] + list(settings.INSTALLED_APPS)
        super(AppResolutionOrderI18NTests, self).setUp()

    def tearDown(self):
        settings.INSTALLED_APPS = self.old_installed_apps
        super(AppResolutionOrderI18NTests, self).tearDown()

    def test_app_translation(self):
        self.assertUgettext('Date/time', 'APP')

class LocalePathsResolutionOrderI18NTests(ResolutionOrderI18NTests):

    def setUp(self):
        self.old_locale_paths = settings.LOCALE_PATHS
        settings.LOCALE_PATHS += (os.path.join(here, 'other', 'locale'),)
        super(LocalePathsResolutionOrderI18NTests, self).setUp()

    def tearDown(self):
        settings.LOCALE_PATHS = self.old_locale_paths
        super(LocalePathsResolutionOrderI18NTests, self).tearDown()

    def test_locale_paths_translation(self):
        self.assertUgettext('Time', 'LOCALE_PATHS')

    def test_locale_paths_override_app_translation(self):
        extended_apps = list(settings.INSTALLED_APPS) + ['regressiontests.i18n.resolution']
        with self.settings(INSTALLED_APPS=extended_apps):
            self.assertUgettext('Time', 'LOCALE_PATHS')

    def test_locale_paths_override_project_translation(self):
        with self.settings(SETTINGS_MODULE='regressiontests'):
            self.assertUgettext('Date/time', 'LOCALE_PATHS')

class ProjectResolutionOrderI18NTests(ResolutionOrderI18NTests):

    def setUp(self):
        self.old_settings_module = settings.SETTINGS_MODULE
        settings.SETTINGS_MODULE = 'regressiontests'
        super(ProjectResolutionOrderI18NTests, self).setUp()

    def tearDown(self):
        settings.SETTINGS_MODULE = self.old_settings_module
        super(ProjectResolutionOrderI18NTests, self).tearDown()

    def test_project_translation(self):
        self.assertUgettext('Date/time', 'PROJECT')

    def test_project_override_app_translation(self):
        extended_apps = list(settings.INSTALLED_APPS) + ['regressiontests.i18n.resolution']
        with self.settings(INSTALLED_APPS=extended_apps):
            self.assertUgettext('Date/time', 'PROJECT')

class DjangoFallbackResolutionOrderI18NTests(ResolutionOrderI18NTests):

    def test_django_fallback(self):
        self.assertEqual(ugettext('Date/time'), 'Datum/Zeit')


class TestModels(TestCase):
    def test_lazy(self):
        tm = TestModel()
        tm.save()

    def test_safestr(self):
        c = Company(cents_paid=12, products_delivered=1)
        c.name = SafeUnicode(u'Iñtërnâtiônàlizætiøn1')
        c.save()
        c.name = SafeString(u'Iñtërnâtiônàlizætiøn1'.encode('utf-8'))
        c.save()


class TestLanguageInfo(TestCase):
    def test_localized_language_info(self):
        li = get_language_info('de')
        self.assertEqual(li['code'], 'de')
        self.assertEqual(li['name_local'], u'Deutsch')
        self.assertEqual(li['name'], 'German')
        self.assertEqual(li['bidi'], False)


class MultipleLocaleActivationTests(TestCase):
    """
    Tests for template rendering behavior when multiple locales are activated
    during the lifetime of the same process.
    """
    def setUp(self):
        self._old_language = get_language()

    def tearDown(self):
        activate(self._old_language)

    def test_single_locale_activation(self):
        """
        Simple baseline behavior with one locale for all the supported i18n constructs.
        """
        with translation.override('fr'):
            self.assertEqual(Template("{{ _('Yes') }}").render(Context({})), 'Oui')
            self.assertEqual(Template("{% load i18n %}{% trans 'Yes' %}").render(Context({})), 'Oui')
            self.assertEqual(Template("{% load i18n %}{% blocktrans %}Yes{% endblocktrans %}").render(Context({})), 'Oui')

    # Literal marked up with _() in a filter expression

    def test_multiple_locale_filter(self):
        with translation.override('de'):
            t = Template("{% load i18n %}{{ 0|yesno:_('yes,no,maybe') }}")
        with translation.override(self._old_language):
            with translation.override('nl'):
                self.assertEqual(t.render(Context({})), 'nee')

    def test_multiple_locale_filter_deactivate(self):
        with translation.override('de', deactivate=True):
            t = Template("{% load i18n %}{{ 0|yesno:_('yes,no,maybe') }}")
        with translation.override('nl'):
            self.assertEqual(t.render(Context({})), 'nee')

    def test_multiple_locale_filter_direct_switch(self):
        with translation.override('de'):
            t = Template("{% load i18n %}{{ 0|yesno:_('yes,no,maybe') }}")
        with translation.override('nl'):
            self.assertEqual(t.render(Context({})), 'nee')

    # Literal marked up with _()

    def test_multiple_locale(self):
        with translation.override('de'):
            t = Template("{{ _('No') }}")
        with translation.override(self._old_language):
            with translation.override('nl'):
                self.assertEqual(t.render(Context({})), 'Nee')

    def test_multiple_locale_deactivate(self):
        with translation.override('de', deactivate=True):
            t = Template("{{ _('No') }}")
        with translation.override('nl'):
            self.assertEqual(t.render(Context({})), 'Nee')

    def test_multiple_locale_direct_switch(self):
        with translation.override('de'):
            t = Template("{{ _('No') }}")
        with translation.override('nl'):
            self.assertEqual(t.render(Context({})), 'Nee')

    # Literal marked up with _(), loading the i18n template tag library

    def test_multiple_locale_loadi18n(self):
        with translation.override('de'):
            t = Template("{% load i18n %}{{ _('No') }}")
        with translation.override(self._old_language):
            with translation.override('nl'):
                self.assertEqual(t.render(Context({})), 'Nee')

    def test_multiple_locale_loadi18n_deactivate(self):
        with translation.override('de', deactivate=True):
            t = Template("{% load i18n %}{{ _('No') }}")
        with translation.override('nl'):
            self.assertEqual(t.render(Context({})), 'Nee')

    def test_multiple_locale_loadi18n_direct_switch(self):
        with translation.override('de'):
            t = Template("{% load i18n %}{{ _('No') }}")
        with translation.override('nl'):
            self.assertEqual(t.render(Context({})), 'Nee')

    # trans i18n tag

    def test_multiple_locale_trans(self):
        with translation.override('de'):
            t = Template("{% load i18n %}{% trans 'No' %}")
        with translation.override(self._old_language):
            with translation.override('nl'):
                self.assertEqual(t.render(Context({})), 'Nee')

    def test_multiple_locale_deactivate_trans(self):
        with translation.override('de', deactivate=True):
            t = Template("{% load i18n %}{% trans 'No' %}")
        with translation.override('nl'):
            self.assertEqual(t.render(Context({})), 'Nee')

    def test_multiple_locale_direct_switch_trans(self):
        with translation.override('de'):
            t = Template("{% load i18n %}{% trans 'No' %}")
        with translation.override('nl'):
            self.assertEqual(t.render(Context({})), 'Nee')

    # blocktrans i18n tag

    def test_multiple_locale_btrans(self):
        with translation.override('de'):
            t = Template("{% load i18n %}{% blocktrans %}No{% endblocktrans %}")
        with translation.override(self._old_language):
            with translation.override('nl'):
                self.assertEqual(t.render(Context({})), 'Nee')

    def test_multiple_locale_deactivate_btrans(self):
        with translation.override('de', deactivate=True):
            t = Template("{% load i18n %}{% blocktrans %}No{% endblocktrans %}")
        with translation.override('nl'):
            self.assertEqual(t.render(Context({})), 'Nee')

    def test_multiple_locale_direct_switch_btrans(self):
        with translation.override('de'):
            t = Template("{% load i18n %}{% blocktrans %}No{% endblocktrans %}")
        with translation.override('nl'):
            self.assertEqual(t.render(Context({})), 'Nee')



"""
Cross Site Request Forgery Middleware.

This module provides a middleware that implements protection
against request forgeries from other sites.
"""

import hashlib
import re
import random

from django.conf import settings
from django.core.urlresolvers import get_callable
from django.utils.cache import patch_vary_headers
from django.utils.http import same_origin
from django.utils.log import getLogger
from django.utils.crypto import constant_time_compare, get_random_string

logger = getLogger('django.request')

REASON_NO_REFERER = "Referer checking failed - no Referer."
REASON_BAD_REFERER = "Referer checking failed - %s does not match %s."
REASON_NO_CSRF_COOKIE = "CSRF cookie not set."
REASON_BAD_TOKEN = "CSRF token missing or incorrect."

CSRF_KEY_LENGTH = 32

def _get_failure_view():
    """
    Returns the view to be used for CSRF rejections
    """
    return get_callable(settings.CSRF_FAILURE_VIEW)


def _get_new_csrf_key():
    return get_random_string(CSRF_KEY_LENGTH)


def get_token(request):
    """
    Returns the the CSRF token required for a POST form. The token is an
    alphanumeric value.

    A side effect of calling this function is to make the the csrf_protect
    decorator and the CsrfViewMiddleware add a CSRF cookie and a 'Vary: Cookie'
    header to the outgoing response.  For this reason, you may need to use this
    function lazily, as is done by the csrf context processor.
    """
    request.META["CSRF_COOKIE_USED"] = True
    return request.META.get("CSRF_COOKIE", None)


def _sanitize_token(token):
    # Allow only alphanum, and ensure we return a 'str' for the sake
    # of the post processing middleware.
    if len(token) > CSRF_KEY_LENGTH:
        return _get_new_csrf_key()
    token = re.sub('[^a-zA-Z0-9]+', '', str(token.decode('ascii', 'ignore')))
    if token == "":
        # In case the cookie has been truncated to nothing at some point.
        return _get_new_csrf_key()
    return token


class CsrfViewMiddleware(object):
    """
    Middleware that requires a present and correct csrfmiddlewaretoken
    for POST requests that have a CSRF cookie, and sets an outgoing
    CSRF cookie.

    This middleware should be used in conjunction with the csrf_token template
    tag.
    """
    # The _accept and _reject methods currently only exist for the sake of the
    # requires_csrf_token decorator.
    def _accept(self, request):
        # Avoid checking the request twice by adding a custom attribute to
        # request.  This will be relevant when both decorator and middleware
        # are used.
        request.csrf_processing_done = True
        return None

    def _reject(self, request, reason):
        return _get_failure_view()(request, reason=reason)

    def process_view(self, request, callback, callback_args, callback_kwargs):

        if getattr(request, 'csrf_processing_done', False):
            return None

        try:
            csrf_token = _sanitize_token(
                request.COOKIES[settings.CSRF_COOKIE_NAME])
            # Use same token next time
            request.META['CSRF_COOKIE'] = csrf_token
        except KeyError:
            csrf_token = None
            # Generate token and store it in the request, so it's
            # available to the view.
            request.META["CSRF_COOKIE"] = _get_new_csrf_key()

        # Wait until request.META["CSRF_COOKIE"] has been manipulated before
        # bailing out, so that get_token still works
        if getattr(callback, 'csrf_exempt', False):
            return None

        # Assume that anything not defined as 'safe' by RC2616 needs protection
        if request.method not in ('GET', 'HEAD', 'OPTIONS', 'TRACE'):
            if getattr(request, '_dont_enforce_csrf_checks', False):
                # Mechanism to turn off CSRF checks for test suite.
                # It comes after the creation of CSRF cookies, so that
                # everything else continues to work exactly the same
                # (e.g. cookies are sent, etc.), but before any
                # branches that call reject().
                return self._accept(request)

            if request.is_secure():
                # Suppose user visits http://example.com/
                # An active network attacker (man-in-the-middle, MITM) sends a
                # POST form that targets https://example.com/detonate-bomb/ and
                # submits it via JavaScript.
                #
                # The attacker will need to provide a CSRF cookie and token, but
                # that's no problem for a MITM and the session-independent
                # nonce we're using. So the MITM can circumvent the CSRF
                # protection. This is true for any HTTP connection, but anyone
                # using HTTPS expects better! For this reason, for
                # https://example.com/ we need additional protection that treats
                # http://example.com/ as completely untrusted. Under HTTPS,
                # Barth et al. found that the Referer header is missing for
                # same-domain requests in only about 0.2% of cases or less, so
                # we can use strict Referer checking.
                referer = request.META.get('HTTP_REFERER')
                if referer is None:
                    logger.warning('Forbidden (%s): %s',
                                   REASON_NO_REFERER, request.path,
                        extra={
                            'status_code': 403,
                            'request': request,
                        }
                    )
                    return self._reject(request, REASON_NO_REFERER)

                # Note that request.get_host() includes the port.
                good_referer = 'https://%s/' % request.get_host()
                if not same_origin(referer, good_referer):
                    reason = REASON_BAD_REFERER % (referer, good_referer)
                    logger.warning('Forbidden (%s): %s', reason, request.path,
                        extra={
                            'status_code': 403,
                            'request': request,
                        }
                    )
                    return self._reject(request, reason)

            if csrf_token is None:
                # No CSRF cookie. For POST requests, we insist on a CSRF cookie,
                # and in this way we can avoid all CSRF attacks, including login
                # CSRF.
                logger.warning('Forbidden (%s): %s',
                               REASON_NO_CSRF_COOKIE, request.path,
                    extra={
                        'status_code': 403,
                        'request': request,
                    }
                )
                return self._reject(request, REASON_NO_CSRF_COOKIE)

            # Check non-cookie token for match.
            request_csrf_token = ""
            if request.method == "POST":
                request_csrf_token = request.POST.get('csrfmiddlewaretoken', '')

            if request_csrf_token == "":
                # Fall back to X-CSRFToken, to make things easier for AJAX,
                # and possible for PUT/DELETE.
                request_csrf_token = request.META.get('HTTP_X_CSRFTOKEN', '')

            if not constant_time_compare(request_csrf_token, csrf_token):
                logger.warning('Forbidden (%s): %s',
                               REASON_BAD_TOKEN, request.path,
                    extra={
                        'status_code': 403,
                        'request': request,
                    }
                )
                return self._reject(request, REASON_BAD_TOKEN)

        return self._accept(request)

    def process_response(self, request, response):
        if getattr(response, 'csrf_processing_done', False):
            return response

        # If CSRF_COOKIE is unset, then CsrfViewMiddleware.process_view was
        # never called, probaby because a request middleware returned a response
        # (for example, contrib.auth redirecting to a login page).
        if request.META.get("CSRF_COOKIE") is None:
            return response

        if not request.META.get("CSRF_COOKIE_USED", False):
            return response

        # Set the CSRF cookie even if it's already set, so we renew
        # the expiry timer.
        response.set_cookie(settings.CSRF_COOKIE_NAME,
                            request.META["CSRF_COOKIE"],
                            max_age = 60 * 60 * 24 * 7 * 52,
                            domain=settings.CSRF_COOKIE_DOMAIN,
                            path=settings.CSRF_COOKIE_PATH,
                            secure=settings.CSRF_COOKIE_SECURE
                            )
        # Content varies with the CSRF cookie, so set the Vary header.
        patch_vary_headers(response, ('Cookie',))
        response.csrf_processing_done = True
        return response



import fnmatch
import glob
import os
import re
import sys
from itertools import dropwhile
from optparse import make_option
from subprocess import PIPE, Popen

import django
from django.core.management.base import CommandError, NoArgsCommand
from django.utils.text import get_text_list
from django.utils.jslex import prepare_js_for_gettext

plural_forms_re = re.compile(r'^(?P<value>"Plural-Forms.+?\\n")\s*$', re.MULTILINE | re.DOTALL)

def handle_extensions(extensions=('html',), ignored=('py',)):
    """
    Organizes multiple extensions that are separated with commas or passed by
    using --extension/-e multiple times. Note that the .py extension is ignored
    here because of the way non-*.py files are handled in make_messages() (they
    are copied to file.ext.py files to trick xgettext to parse them as Python
    files).

    For example: running 'django-admin makemessages -e js,txt -e xhtml -a'
    would result in an extension list: ['.js', '.txt', '.xhtml']

    >>> handle_extensions(['.html', 'html,js,py,py,py,.py', 'py,.py'])
    set(['.html', '.js'])
    >>> handle_extensions(['.html, txt,.tpl'])
    set(['.html', '.tpl', '.txt'])
    """
    ext_list = []
    for ext in extensions:
        ext_list.extend(ext.replace(' ', '').split(','))
    for i, ext in enumerate(ext_list):
        if not ext.startswith('.'):
            ext_list[i] = '.%s' % ext_list[i]
    return set([x for x in ext_list if x.strip('.') not in ignored])

def _popen(cmd):
    """
    Friendly wrapper around Popen for Windows
    """
    p = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE, close_fds=os.name != 'nt', universal_newlines=True)
    return p.communicate()

def walk(root, topdown=True, onerror=None, followlinks=False,
         ignore_patterns=None, verbosity=0, stdout=sys.stdout):
    """
    A version of os.walk that can follow symlinks for Python < 2.6
    """
    if ignore_patterns is None:
        ignore_patterns = []
    dir_suffix = '%s*' % os.sep
    norm_patterns = map(lambda p: p.endswith(dir_suffix)
                        and p[:-len(dir_suffix)] or p, ignore_patterns)
    for dirpath, dirnames, filenames in os.walk(root, topdown, onerror):
        remove_dirs = []
        for dirname in dirnames:
            if is_ignored(os.path.normpath(os.path.join(dirpath, dirname)), norm_patterns):
                remove_dirs.append(dirname)
        for dirname in remove_dirs:
            dirnames.remove(dirname)
            if verbosity > 1:
                stdout.write('ignoring directory %s\n' % dirname)
        yield (dirpath, dirnames, filenames)
        if followlinks:
            for d in dirnames:
                p = os.path.join(dirpath, d)
                if os.path.islink(p):
                    for link_dirpath, link_dirnames, link_filenames in walk(p):
                        yield (link_dirpath, link_dirnames, link_filenames)

def is_ignored(path, ignore_patterns):
    """
    Helper function to check if the given path should be ignored or not.
    """
    for pattern in ignore_patterns:
        if fnmatch.fnmatchcase(path, pattern):
            return True
    return False

def find_files(root, ignore_patterns, verbosity, stdout=sys.stdout, symlinks=False):
    """
    Helper function to get all files in the given root.
    """
    all_files = []
    for (dirpath, dirnames, filenames) in walk(root, followlinks=symlinks,
            ignore_patterns=ignore_patterns, verbosity=verbosity, stdout=stdout):
        for filename in filenames:
            norm_filepath = os.path.normpath(os.path.join(dirpath, filename))
            if is_ignored(norm_filepath, ignore_patterns):
                if verbosity > 1:
                    stdout.write('ignoring file %s in %s\n' % (filename, dirpath))
            else:
                all_files.extend([(dirpath, filename)])
    all_files.sort()
    return all_files

def copy_plural_forms(msgs, locale, domain, verbosity, stdout=sys.stdout):
    """
    Copies plural forms header contents from a Django catalog of locale to
    the msgs string, inserting it at the right place. msgs should be the
    contents of a newly created .po file.
    """
    django_dir = os.path.normpath(os.path.join(os.path.dirname(django.__file__)))
    if domain == 'djangojs':
        domains = ('djangojs', 'django')
    else:
        domains = ('django',)
    for domain in domains:
        django_po = os.path.join(django_dir, 'conf', 'locale', locale, 'LC_MESSAGES', '%s.po' % domain)
        if os.path.exists(django_po):
            m = plural_forms_re.search(open(django_po, 'rU').read())
            if m:
                if verbosity > 1:
                    stdout.write("copying plural forms: %s\n" % m.group('value'))
                lines = []
                seen = False
                for line in msgs.split('\n'):
                    if not line and not seen:
                        line = '%s\n' % m.group('value')
                        seen = True
                    lines.append(line)
                msgs = '\n'.join(lines)
                break
    return msgs

def write_pot_file(potfile, msgs, file, work_file, is_templatized):
    """
    Write the :param potfile: POT file with the :param msgs: contents,
    previously making sure its format is valid.
    """
    if is_templatized:
        old = '#: ' + work_file[2:]
        new = '#: ' + file[2:]
        msgs = msgs.replace(old, new)
    if os.path.exists(potfile):
        # Strip the header
        msgs = '\n'.join(dropwhile(len, msgs.split('\n')))
    else:
        msgs = msgs.replace('charset=CHARSET', 'charset=UTF-8')
    f = open(potfile, 'ab')
    try:
        f.write(msgs)
    finally:
        f.close()

def process_file(file, dirpath, potfile, domain, verbosity,
                 extensions, wrap, location, stdout=sys.stdout):
    """
    Extract translatable literals from :param file: for :param domain:
    creating or updating the :param potfile: POT file.

    Uses the xgettext GNU gettext utility.
    """

    from django.utils.translation import templatize

    if verbosity > 1:
        stdout.write('processing file %s in %s\n' % (file, dirpath))
    _, file_ext = os.path.splitext(file)
    if domain == 'djangojs' and file_ext in extensions:
        is_templatized = True
        orig_file = os.path.join(dirpath, file)
        src_data = open(orig_file).read()
        src_data = prepare_js_for_gettext(src_data)
        thefile = '%s.c' % file
        work_file = os.path.join(dirpath, thefile)
        f = open(work_file, "w")
        try:
            f.write(src_data)
        finally:
            f.close()
        cmd = (
            'xgettext -d %s -L C %s %s --keyword=gettext_noop '
            '--keyword=gettext_lazy --keyword=ngettext_lazy:1,2 '
            '--keyword=pgettext:1c,2 --keyword=npgettext:1c,2,3 '
            '--from-code UTF-8 --add-comments=Translators -o - "%s"' %
            (domain, wrap, location, work_file))
    elif domain == 'django' and (file_ext == '.py' or file_ext in extensions):
        thefile = file
        orig_file = os.path.join(dirpath, file)
        is_templatized = file_ext in extensions
        if is_templatized:
            src_data = open(orig_file, "rU").read()
            thefile = '%s.py' % file
            content = templatize(src_data, orig_file[2:])
            f = open(os.path.join(dirpath, thefile), "w")
            try:
                f.write(content)
            finally:
                f.close()
        work_file = os.path.join(dirpath, thefile)
        cmd = (
            'xgettext -d %s -L Python %s %s --keyword=gettext_noop '
            '--keyword=gettext_lazy --keyword=ngettext_lazy:1,2 '
            '--keyword=ugettext_noop --keyword=ugettext_lazy '
            '--keyword=ungettext_lazy:1,2 --keyword=pgettext:1c,2 '
            '--keyword=npgettext:1c,2,3 --keyword=pgettext_lazy:1c,2 '
            '--keyword=npgettext_lazy:1c,2,3 --from-code UTF-8 '
            '--add-comments=Translators -o - "%s"' %
            (domain, wrap, location, work_file))
    else:
        return
    msgs, errors = _popen(cmd)
    if errors:
        if is_templatized:
            os.unlink(work_file)
        if os.path.exists(potfile):
            os.unlink(potfile)
        raise CommandError(
            "errors happened while running xgettext on %s\n%s" %
            (file, errors))
    if msgs:
        write_pot_file(potfile, msgs, orig_file, work_file, is_templatized)
    if is_templatized:
        os.unlink(work_file)

def write_po_file(pofile, potfile, domain, locale, verbosity, stdout,
                  copy_pforms, wrap, location, no_obsolete):
    """
    Creates of updates the :param pofile: PO file for :param domain: and :param
    locale:.  Uses contents of the existing :param potfile:.

    Uses mguniq, msgmerge, and msgattrib GNU gettext utilities.
    """
    msgs, errors = _popen('msguniq %s %s --to-code=utf-8 "%s"' %
                            (wrap, location, potfile))
    if errors:
        os.unlink(potfile)
        raise CommandError("errors happened while running msguniq\n%s" % errors)
    if os.path.exists(pofile):
        f = open(potfile, 'w')
        try:
            f.write(msgs)
        finally:
            f.close()
        msgs, errors = _popen('msgmerge %s %s -q "%s" "%s"' %
                                (wrap, location, pofile, potfile))
        if errors:
            os.unlink(potfile)
            raise CommandError(
                "errors happened while running msgmerge\n%s" % errors)
    elif copy_pforms:
        msgs = copy_plural_forms(msgs, locale, domain, verbosity, stdout)
    msgs = msgs.replace(
        "#. #-#-#-#-#  %s.pot (PACKAGE VERSION)  #-#-#-#-#\n" % domain, "")
    f = open(pofile, 'wb')
    try:
        f.write(msgs)
    finally:
        f.close()
    os.unlink(potfile)
    if no_obsolete:
        msgs, errors = _popen('msgattrib %s %s -o "%s" --no-obsolete "%s"' %
                                (wrap, location, pofile, pofile))
        if errors:
            raise CommandError(
                "errors happened while running msgattrib\n%s" % errors)

def make_messages(locale=None, domain='django', verbosity=1, all=False,
        extensions=None, symlinks=False, ignore_patterns=None, no_wrap=False,
        no_location=False, no_obsolete=False, stdout=sys.stdout):
    """
    Uses the ``locale/`` directory from the Django SVN tree or an
    application/project to process all files with translatable literals for
    the :param domain: domain and :param locale: locale.
    """
    # Need to ensure that the i18n framework is enabled
    from django.conf import settings
    if settings.configured:
        settings.USE_I18N = True
    else:
        settings.configure(USE_I18N = True)

    if ignore_patterns is None:
        ignore_patterns = []

    invoked_for_django = False
    if os.path.isdir(os.path.join('conf', 'locale')):
        localedir = os.path.abspath(os.path.join('conf', 'locale'))
        invoked_for_django = True
        # Ignoring all contrib apps
        ignore_patterns += ['contrib/*']
    elif os.path.isdir('locale'):
        localedir = os.path.abspath('locale')
    else:
        raise CommandError("This script should be run from the Django SVN "
                "tree or your project or app tree. If you did indeed run it "
                "from the SVN checkout or your project or application, "
                "maybe you are just missing the conf/locale (in the django "
                "tree) or locale (for project and application) directory? It "
                "is not created automatically, you have to create it by hand "
                "if you want to enable i18n for your project or application.")

    if domain not in ('django', 'djangojs'):
        raise CommandError("currently makemessages only supports domains 'django' and 'djangojs'")

    if (locale is None and not all) or domain is None:
        message = "Type '%s help %s' for usage information." % (os.path.basename(sys.argv[0]), sys.argv[1])
        raise CommandError(message)

    # We require gettext version 0.15 or newer.
    output = _popen('xgettext --version')[0]
    match = re.search(r'(?P<major>\d+)\.(?P<minor>\d+)', output)
    if match:
        xversion = (int(match.group('major')), int(match.group('minor')))
        if xversion < (0, 15):
            raise CommandError("Django internationalization requires GNU "
                    "gettext 0.15 or newer. You are using version %s, please "
                    "upgrade your gettext toolset." % match.group())

    locales = []
    if locale is not None:
        locales.append(locale)
    elif all:
        locale_dirs = filter(os.path.isdir, glob.glob('%s/*' % localedir))
        locales = [os.path.basename(l) for l in locale_dirs]

    wrap = '--no-wrap' if no_wrap else ''
    location = '--no-location' if no_location else ''

    for locale in locales:
        if verbosity > 0:
            stdout.write("processing language %s\n" % locale)
        basedir = os.path.join(localedir, locale, 'LC_MESSAGES')
        if not os.path.isdir(basedir):
            os.makedirs(basedir)

        pofile = os.path.join(basedir, '%s.po' % domain)
        potfile = os.path.join(basedir, '%s.pot' % domain)

        if os.path.exists(potfile):
            os.unlink(potfile)

        for dirpath, file in find_files(".", ignore_patterns, verbosity,
                stdout, symlinks=symlinks):
            process_file(file, dirpath, potfile, domain, verbosity, extensions,
                    wrap, location, stdout)

        if os.path.exists(potfile):
            write_po_file(pofile, potfile, domain, locale, verbosity, stdout,
                    not invoked_for_django, wrap, location, no_obsolete)


class Command(NoArgsCommand):
    option_list = NoArgsCommand.option_list + (
        make_option('--locale', '-l', default=None, dest='locale',
            help='Creates or updates the message files for the given locale (e.g. pt_BR).'),
        make_option('--domain', '-d', default='django', dest='domain',
            help='The domain of the message files (default: "django").'),
        make_option('--all', '-a', action='store_true', dest='all',
            default=False, help='Updates the message files for all existing locales.'),
        make_option('--extension', '-e', dest='extensions',
            help='The file extension(s) to examine (default: "html,txt", or "js" if the domain is "djangojs"). Separate multiple extensions with commas, or use -e multiple times.',
            action='append'),
        make_option('--symlinks', '-s', action='store_true', dest='symlinks',
            default=False, help='Follows symlinks to directories when examining source code and templates for translation strings.'),
        make_option('--ignore', '-i', action='append', dest='ignore_patterns',
            default=[], metavar='PATTERN', help='Ignore files or directories matching this glob-style pattern. Use multiple times to ignore more.'),
        make_option('--no-default-ignore', action='store_false', dest='use_default_ignore_patterns',
            default=True, help="Don't ignore the common glob-style patterns 'CVS', '.*' and '*~'."),
        make_option('--no-wrap', action='store_true', dest='no_wrap',
            default=False, help="Don't break long message lines into several lines"),
        make_option('--no-location', action='store_true', dest='no_location',
            default=False, help="Don't write '#: filename:line' lines"),
        make_option('--no-obsolete', action='store_true', dest='no_obsolete',
            default=False, help="Remove obsolete message strings"),
    )
    help = ("Runs over the entire source tree of the current directory and "
"pulls out all strings marked for translation. It creates (or updates) a message "
"file in the conf/locale (in the django tree) or locale (for projects and "
"applications) directory.\n\nYou must run this command with one of either the "
"--locale or --all options.")

    requires_model_validation = False
    can_import_settings = False

    def handle_noargs(self, *args, **options):
        locale = options.get('locale')
        domain = options.get('domain')
        verbosity = int(options.get('verbosity'))
        process_all = options.get('all')
        extensions = options.get('extensions')
        symlinks = options.get('symlinks')
        ignore_patterns = options.get('ignore_patterns')
        if options.get('use_default_ignore_patterns'):
            ignore_patterns += ['CVS', '.*', '*~']
        ignore_patterns = list(set(ignore_patterns))
        no_wrap = options.get('no_wrap')
        no_location = options.get('no_location')
        no_obsolete = options.get('no_obsolete')
        if domain == 'djangojs':
            exts = extensions if extensions else ['js']
        else:
            exts = extensions if extensions else ['html', 'txt']
        extensions = handle_extensions(exts)

        if verbosity > 1:
            self.stdout.write('examining files with the extensions: %s\n'
                             % get_text_list(list(extensions), 'and'))

        make_messages(locale, domain, verbosity, process_all, extensions,
            symlinks, ignore_patterns, no_wrap, no_location, no_obsolete, self.stdout)



"""
Tests for geography support in PostGIS 1.5+
"""
from __future__ import absolute_import

import os

from django.contrib.gis import gdal
from django.contrib.gis.measure import D
from django.test import TestCase

from .models import City, County, Zipcode


class GeographyTest(TestCase):

    def test01_fixture_load(self):
        "Ensure geography features loaded properly."
        self.assertEqual(8, City.objects.count())

    def test02_distance_lookup(self):
        "Testing GeoQuerySet distance lookup support on non-point geography fields."
        z = Zipcode.objects.get(code='77002')
        cities1 = list(City.objects
                       .filter(point__distance_lte=(z.poly, D(mi=500)))
                       .order_by('name')
                       .values_list('name', flat=True))
        cities2 = list(City.objects
                       .filter(point__dwithin=(z.poly, D(mi=500)))
                       .order_by('name')
                       .values_list('name', flat=True))
        for cities in [cities1, cities2]:
            self.assertEqual(['Dallas', 'Houston', 'Oklahoma City'], cities)

    def test03_distance_method(self):
        "Testing GeoQuerySet.distance() support on non-point geography fields."
        # `GeoQuerySet.distance` is not allowed geometry fields.
        htown = City.objects.get(name='Houston')
        qs = Zipcode.objects.distance(htown.point)

    def test04_invalid_operators_functions(self):
        "Ensuring exceptions are raised for operators & functions invalid on geography fields."
        # Only a subset of the geometry functions & operator are available
        # to PostGIS geography types.  For more information, visit:
        #  http://postgis.refractions.net/documentation/manual-1.5/ch08.html#PostGIS_GeographyFunctions
        z = Zipcode.objects.get(code='77002')
        # ST_Within not available.
        self.assertRaises(ValueError, City.objects.filter(point__within=z.poly).count)
        # `@` operator not available.
        self.assertRaises(ValueError, City.objects.filter(point__contained=z.poly).count)

        # Regression test for #14060, `~=` was never really implemented for PostGIS.
        htown = City.objects.get(name='Houston')
        self.assertRaises(ValueError, City.objects.get, point__exact=htown.point)

    def test05_geography_layermapping(self):
        "Testing LayerMapping support on models with geography fields."
        # There is a similar test in `layermap` that uses the same data set,
        # but the County model here is a bit different.
        if not gdal.HAS_GDAL: return
        from django.contrib.gis.utils import LayerMapping

        # Getting the shapefile and mapping dictionary.
        shp_path = os.path.realpath(os.path.join(os.path.dirname(__file__), '..', 'data'))
        co_shp = os.path.join(shp_path, 'counties', 'counties.shp')
        co_mapping = {'name' : 'Name',
                      'state' : 'State',
                      'mpoly' : 'MULTIPOLYGON',
                      }

        # Reference county names, number of polygons, and state names.
        names = ['Bexar', 'Galveston', 'Harris', 'Honolulu', 'Pueblo']
        num_polys = [1, 2, 1, 19, 1] # Number of polygons for each.
        st_names = ['Texas', 'Texas', 'Texas', 'Hawaii', 'Colorado']

        lm = LayerMapping(County, co_shp, co_mapping, source_srs=4269, unique='name')
        lm.save(silent=True, strict=True)

        for c, name, num_poly, state in zip(County.objects.order_by('name'), names, num_polys, st_names):
            self.assertEqual(4326, c.mpoly.srid)
            self.assertEqual(num_poly, len(c.mpoly))
            self.assertEqual(name, c.name)
            self.assertEqual(state, c.state)

    def test06_geography_area(self):
        "Testing that Area calculations work on geography columns."
        # SELECT ST_Area(poly) FROM geogapp_zipcode WHERE code='77002';
        ref_area = 5439084.70637573
        tol = 5
        z = Zipcode.objects.area().get(code='77002')
        self.assertAlmostEqual(z.area.sq_m, ref_area, tol)



import platform
import re
import urllib
import urllib2
import urlparse

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import smart_unicode
from django.utils.ipv6 import is_valid_ipv6_address

# These values, if given to validate(), will trigger the self.required check.
EMPTY_VALUES = (None, '', [], (), {})

try:
    from django.conf import settings
    URL_VALIDATOR_USER_AGENT = settings.URL_VALIDATOR_USER_AGENT
except ImportError:
    # It's OK if Django settings aren't configured.
    URL_VALIDATOR_USER_AGENT = 'Django (http://www.djangoproject.com/)'

class RegexValidator(object):
    regex = ''
    message = _(u'Enter a valid value.')
    code = 'invalid'

    def __init__(self, regex=None, message=None, code=None):
        if regex is not None:
            self.regex = regex
        if message is not None:
            self.message = message
        if code is not None:
            self.code = code

        # Compile the regex if it was not passed pre-compiled.
        if isinstance(self.regex, basestring):
            self.regex = re.compile(self.regex)

    def __call__(self, value):
        """
        Validates that the input matches the regular expression.
        """
        if not self.regex.search(smart_unicode(value)):
            raise ValidationError(self.message, code=self.code)

class URLValidator(RegexValidator):
    regex = re.compile(
        r'^(?:http|ftp)s?://' # http:// or https://
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|' #domain...
        r'localhost|' #localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})' # ...or ip
        r'(?::\d+)?' # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)

    def __init__(self, verify_exists=False,
                 validator_user_agent=URL_VALIDATOR_USER_AGENT):
        super(URLValidator, self).__init__()
        self.verify_exists = verify_exists
        self.user_agent = validator_user_agent

    def __call__(self, value):
        try:
            super(URLValidator, self).__call__(value)
        except ValidationError, e:
            # Trivial case failed. Try for possible IDN domain
            if value:
                value = smart_unicode(value)
                scheme, netloc, path, query, fragment = urlparse.urlsplit(value)
                try:
                    netloc = netloc.encode('idna') # IDN -> ACE
                except UnicodeError: # invalid domain part
                    raise e
                url = urlparse.urlunsplit((scheme, netloc, path, query, fragment))
                super(URLValidator, self).__call__(url)
            else:
                raise
        else:
            url = value

        if self.verify_exists:
            import warnings
            warnings.warn(
                "The URLField verify_exists argument has intractable security "
                "and performance issues. Accordingly, it has been deprecated.",
                DeprecationWarning
                )

            headers = {
                "Accept": "text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5",
                "Accept-Language": "en-us,en;q=0.5",
                "Accept-Charset": "ISO-8859-1,utf-8;q=0.7,*;q=0.7",
                "Connection": "close",
                "User-Agent": self.user_agent,
            }
            url = url.encode('utf-8')
            # Quote characters from the unreserved set, refs #16812
            url = urllib.quote(url, "!*'();:@&=+$,/?#[]")
            broken_error = ValidationError(
                _(u'This URL appears to be a broken link.'), code='invalid_link')
            try:
                req = urllib2.Request(url, None, headers)
                req.get_method = lambda: 'HEAD'
                #Create an opener that does not support local file access
                opener = urllib2.OpenerDirector()

                #Don't follow redirects, but don't treat them as errors either
                error_nop = lambda *args, **kwargs: True
                http_error_processor = urllib2.HTTPErrorProcessor()
                http_error_processor.http_error_301 = error_nop
                http_error_processor.http_error_302 = error_nop
                http_error_processor.http_error_307 = error_nop

                handlers = [urllib2.UnknownHandler(),
                            urllib2.HTTPHandler(),
                            urllib2.HTTPDefaultErrorHandler(),
                            urllib2.FTPHandler(),
                            http_error_processor]
                try:
                    import ssl
                except ImportError:
                    # Python isn't compiled with SSL support
                    pass
                else:
                    handlers.append(urllib2.HTTPSHandler())
                map(opener.add_handler, handlers)
                if platform.python_version_tuple() >= (2, 6):
                    opener.open(req, timeout=10)
                else:
                    opener.open(req)
            except ValueError:
                raise ValidationError(_(u'Enter a valid URL.'), code='invalid')
            except: # urllib2.URLError, httplib.InvalidURL, etc.
                raise broken_error


def validate_integer(value):
    try:
        int(value)
    except (ValueError, TypeError):
        raise ValidationError('')

class EmailValidator(RegexValidator):

    def __call__(self, value):
        try:
            super(EmailValidator, self).__call__(value)
        except ValidationError, e:
            # Trivial case failed. Try for possible IDN domain-part
            if value and u'@' in value:
                parts = value.split(u'@')
                try:
                    parts[-1] = parts[-1].encode('idna')
                except UnicodeError:
                    raise e
                super(EmailValidator, self).__call__(u'@'.join(parts))
            else:
                raise

email_re = re.compile(
    r"(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*"  # dot-atom
    # quoted-string, see also http://tools.ietf.org/html/rfc2822#section-3.2.5
    r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-\011\013\014\016-\177])*"'
    r')@((?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?$)'  # domain
    r'|\[(25[0-5]|2[0-4]\d|[0-1]?\d?\d)(\.(25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}\]$', re.IGNORECASE)  # literal form, ipv4 address (SMTP 4.1.3)
validate_email = EmailValidator(email_re, _(u'Enter a valid e-mail address.'), 'invalid')

slug_re = re.compile(r'^[-\w]+$')
validate_slug = RegexValidator(slug_re, _(u"Enter a valid 'slug' consisting of letters, numbers, underscores or hyphens."), 'invalid')

ipv4_re = re.compile(r'^(25[0-5]|2[0-4]\d|[0-1]?\d?\d)(\.(25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}$')
validate_ipv4_address = RegexValidator(ipv4_re, _(u'Enter a valid IPv4 address.'), 'invalid')

def validate_ipv6_address(value):
    if not is_valid_ipv6_address(value):
        raise ValidationError(_(u'Enter a valid IPv6 address.'), code='invalid')

def validate_ipv46_address(value):
    try:
        validate_ipv4_address(value)
    except ValidationError:
        try:
            validate_ipv6_address(value)
        except ValidationError:
            raise ValidationError(_(u'Enter a valid IPv4 or IPv6 address.'), code='invalid')

ip_address_validator_map = {
    'both': ([validate_ipv46_address], _('Enter a valid IPv4 or IPv6 address.')),
    'ipv4': ([validate_ipv4_address], _('Enter a valid IPv4 address.')),
    'ipv6': ([validate_ipv6_address], _('Enter a valid IPv6 address.')),
}

def ip_address_validators(protocol, unpack_ipv4):
    """
    Depending on the given parameters returns the appropriate validators for
    the GenericIPAddressField.

    This code is here, because it is exactly the same for the model and the form field.
    """
    if protocol != 'both' and unpack_ipv4:
        raise ValueError(
            "You can only use `unpack_ipv4` if `protocol` is set to 'both'")
    try:
        return ip_address_validator_map[protocol.lower()]
    except KeyError:
        raise ValueError("The protocol '%s' is unknown. Supported: %s"
                         % (protocol, ip_address_validator_map.keys()))

comma_separated_int_list_re = re.compile('^[\d,]+$')
validate_comma_separated_integer_list = RegexValidator(comma_separated_int_list_re, _(u'Enter only digits separated by commas.'), 'invalid')


class BaseValidator(object):
    compare = lambda self, a, b: a is not b
    clean   = lambda self, x: x
    message = _(u'Ensure this value is %(limit_value)s (it is %(show_value)s).')
    code = 'limit_value'

    def __init__(self, limit_value):
        self.limit_value = limit_value

    def __call__(self, value):
        cleaned = self.clean(value)
        params = {'limit_value': self.limit_value, 'show_value': cleaned}
        if self.compare(cleaned, self.limit_value):
            raise ValidationError(
                self.message % params,
                code=self.code,
                params=params,
            )

class MaxValueValidator(BaseValidator):
    compare = lambda self, a, b: a > b
    message = _(u'Ensure this value is less than or equal to %(limit_value)s.')
    code = 'max_value'

class MinValueValidator(BaseValidator):
    compare = lambda self, a, b: a < b
    message = _(u'Ensure this value is greater than or equal to %(limit_value)s.')
    code = 'min_value'

class MinLengthValidator(BaseValidator):
    compare = lambda self, a, b: a < b
    clean   = lambda self, x: len(x)
    message = _(u'Ensure this value has at least %(limit_value)d characters (it has %(show_value)d).')
    code = 'min_length'

class MaxLengthValidator(BaseValidator):
    compare = lambda self, a, b: a > b
    clean   = lambda self, x: len(x)
    message = _(u'Ensure this value has at most %(limit_value)d characters (it has %(show_value)d).')
    code = 'max_length'




from __future__ import absolute_import

from django.contrib.localflavor.mk.forms import (
    MKIdentityCardNumberField, MKMunicipalitySelect, UMCNField)
from django.test import SimpleTestCase

from .forms import MKPersonForm


class MKLocalFlavorTests(SimpleTestCase):

    def setUp(self):
        self.form = MKPersonForm({
            'first_name':'Someone',
            'last_name':'Something',
            'umcn': '2402983450006',
            'municipality':'OD',
            'municipality_req':'VE',
            'id_number':'A1234567',
        })

    def test_get_display_methods(self):
        """
        Test that the get_*_display() methods are added to the model instances.
        """
        person = self.form.save()
        self.assertEqual(person.get_municipality_display(), 'Ohrid')
        self.assertEqual(person.get_municipality_req_display(), 'Veles')

    def test_municipality_required(self):
        """
        Test that required MKMunicipalityFields throw appropriate errors.
        """

        form = MKPersonForm({
            'first_name':'Someone',
            'last_name':'Something',
            'umcn': '2402983450006',
            'municipality':'OD',
            'id_number':'A1234567',
        })
        self.assertFalse(form.is_valid())
        self.assertEqual(
            form.errors['municipality_req'], [u'This field is required.'])

    def test_umcn_invalid(self):
        """
        Test that UMCNFields throw appropriate errors for invalid UMCNs.
        """
        form = MKPersonForm({
            'first_name':'Someone',
            'last_name':'Something',
            'umcn': '2402983450007',
            'municipality':'OD',
            'municipality_req':'VE',
            'id_number':'A1234567',
        })
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors['umcn'], [u'The UMCN is not valid.'])

        form = MKPersonForm({
            'first_name':'Someone',
            'last_name':'Something',
            'umcn': '3002983450007',
            'municipality':'OD',
            'municipality_req':'VE',
            'id_number':'A1234567',
        })
        self.assertEqual(form.errors['umcn'],
                [u'The first 7 digits of the UMCN must represent a valid past date.'])

    def test_idnumber_invalid(self):
        """
        Test that MKIdentityCardNumberFields throw
        appropriate errors for invalid values
        """

        form = MKPersonForm({
            'first_name':'Someone',
            'last_name':'Something',
            'umcn': '2402983450007',
            'municipality':'OD',
            'municipality_req':'VE',
            'id_number':'A123456a',
        })
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors['id_number'],
            [u'Identity card numbers must contain either 4 to 7 '
              'digits or an uppercase letter and 7 digits.'])

    def test_field_blank_option(self):
        """
        Test that the empty option is there.
        """
        municipality_select_html = """\
<select name="municipality" id="id_municipality">
<option value="">---------</option>
<option value="AD">Aerodrom</option>
<option value="AR">Ara\xc4\x8dinovo</option>
<option value="BR">Berovo</option>
<option value="TL">Bitola</option>
<option value="BG">Bogdanci</option>
<option value="VJ">Bogovinje</option>
<option value="BS">Bosilovo</option>
<option value="BN">Brvenica</option>
<option value="BU">Butel</option>
<option value="VA">Valandovo</option>
<option value="VL">Vasilevo</option>
<option value="VV">Vev\xc4\x8dani</option>
<option value="VE">Veles</option>
<option value="NI">Vinica</option>
<option value="VC">Vrane\xc5\xa1tica</option>
<option value="VH">Vrap\xc4\x8di\xc5\xa1te</option>
<option value="GB">Gazi Baba</option>
<option value="GV">Gevgelija</option>
<option value="GT">Gostivar</option>
<option value="GR">Gradsko</option>
<option value="DB">Debar</option>
<option value="DA">Debarca</option>
<option value="DL">Del\xc4\x8devo</option>
<option value="DK">Demir Kapija</option>
<option value="DM">Demir Hisar</option>
<option value="DE">Dolneni</option>
<option value="DR">Drugovo</option>
<option value="GP">Gjor\xc4\x8de Petrov</option>
<option value="ZE">\xc5\xbdelino</option>
<option value="ZA">Zajas</option>
<option value="ZK">Zelenikovo</option>
<option value="ZR">Zrnovci</option>
<option value="IL">Ilinden</option>
<option value="JG">Jegunovce</option>
<option value="AV">Kavadarci</option>
<option value="KB">Karbinci</option>
<option value="KX">Karpo\xc5\xa1</option>
<option value="VD">Kisela Voda</option>
<option value="KH">Ki\xc4\x8devo</option>
<option value="KN">Kon\xc4\x8de</option>
<option value="OC">Ko\xc4\x87ani</option>
<option value="KY">Kratovo</option>
<option value="KZ">Kriva Palanka</option>
<option value="KG">Krivoga\xc5\xa1tani</option>
<option value="KS">Kru\xc5\xa1evo</option>
<option value="UM">Kumanovo</option>
<option value="LI">Lipkovo</option>
<option value="LO">Lozovo</option>
<option value="MR">Mavrovo i Rostu\xc5\xa1a</option>
<option value="MK">Makedonska Kamenica</option>
<option value="MD">Makedonski Brod</option>
<option value="MG">Mogila</option>
<option value="NG">Negotino</option>
<option value="NV">Novaci</option>
<option value="NS">Novo Selo</option>
<option value="OS">Oslomej</option>
<option value="OD" selected="selected">Ohrid</option>
<option value="PE">Petrovec</option>
<option value="PH">Peh\xc4\x8devo</option>
<option value="PN">Plasnica</option>
<option value="PP">Prilep</option>
<option value="PT">Probi\xc5\xa1tip</option>
<option value="RV">Radovi\xc5\xa1</option>
<option value="RN">Rankovce</option>
<option value="RE">Resen</option>
<option value="RO">Rosoman</option>
<option value="AJ">Saraj</option>
<option value="SL">Sveti Nikole</option>
<option value="SS">Sopi\xc5\xa1te</option>
<option value="SD">Star Dojran</option>
<option value="NA">Staro Nagori\xc4\x8dane</option>
<option value="UG">Struga</option>
<option value="RU">Strumica</option>
<option value="SU">Studeni\xc4\x8dani</option>
<option value="TR">Tearce</option>
<option value="ET">Tetovo</option>
<option value="CE">Centar</option>
<option value="CZ">Centar-\xc5\xbdupa</option>
<option value="CI">\xc4\x8cair</option>
<option value="CA">\xc4\x8ca\xc5\xa1ka</option>
<option value="CH">\xc4\x8ce\xc5\xa1inovo-Oble\xc5\xa1evo</option>
<option value="CS">\xc4\x8cu\xc4\x8der-Sandevo</option>
<option value="ST">\xc5\xa0tip</option>
<option value="SO">\xc5\xa0uto Orizari</option>
</select>"""
        self.assertHTMLEqual(str(self.form['municipality']), municipality_select_html)

    def test_MKIdentityCardNumberField(self):
        error_invalid  = [u'Identity card numbers must contain either 4 to 7 '
                          'digits or an uppercase letter and 7 digits.']
        valid = {
            'L0018077':'L0018077',
            'A0078315' : 'A0078315',
        }
        invalid = {
            '123': error_invalid,
            'abcdf': error_invalid,
            '234390a': error_invalid,
        }
        self.assertFieldOutput(MKIdentityCardNumberField, valid, invalid)

    def test_MKMunicipalitySelect(self):
        f = MKMunicipalitySelect()
        out=u'''<select name="municipality">
<option value="AD">Aerodrom</option>
<option value="AR">Ara\u010dinovo</option>
<option value="BR">Berovo</option>
<option value="TL">Bitola</option>
<option value="BG">Bogdanci</option>
<option value="VJ">Bogovinje</option>
<option value="BS">Bosilovo</option>
<option value="BN">Brvenica</option>
<option value="BU">Butel</option>
<option value="VA">Valandovo</option>
<option value="VL">Vasilevo</option>
<option value="VV">Vev\u010dani</option>
<option value="VE">Veles</option>
<option value="NI">Vinica</option>
<option value="VC">Vrane\u0161tica</option>
<option value="VH">Vrap\u010di\u0161te</option>
<option value="GB">Gazi Baba</option>
<option value="GV">Gevgelija</option>
<option value="GT">Gostivar</option>
<option value="GR">Gradsko</option>
<option value="DB">Debar</option>
<option value="DA">Debarca</option>
<option value="DL" selected="selected">Del\u010devo</option>
<option value="DK">Demir Kapija</option>
<option value="DM">Demir Hisar</option>
<option value="DE">Dolneni</option>
<option value="DR">Drugovo</option>
<option value="GP">Gjor\u010de Petrov</option>
<option value="ZE">\u017delino</option>
<option value="ZA">Zajas</option>
<option value="ZK">Zelenikovo</option>
<option value="ZR">Zrnovci</option>
<option value="IL">Ilinden</option>
<option value="JG">Jegunovce</option>
<option value="AV">Kavadarci</option>
<option value="KB">Karbinci</option>
<option value="KX">Karpo\u0161</option>
<option value="VD">Kisela Voda</option>
<option value="KH">Ki\u010devo</option>
<option value="KN">Kon\u010de</option>
<option value="OC">Ko\u0107ani</option>
<option value="KY">Kratovo</option>
<option value="KZ">Kriva Palanka</option>
<option value="KG">Krivoga\u0161tani</option>
<option value="KS">Kru\u0161evo</option>
<option value="UM">Kumanovo</option>
<option value="LI">Lipkovo</option>
<option value="LO">Lozovo</option>
<option value="MR">Mavrovo i Rostu\u0161a</option>
<option value="MK">Makedonska Kamenica</option>
<option value="MD">Makedonski Brod</option>
<option value="MG">Mogila</option>
<option value="NG">Negotino</option>
<option value="NV">Novaci</option>
<option value="NS">Novo Selo</option>
<option value="OS">Oslomej</option>
<option value="OD">Ohrid</option>
<option value="PE">Petrovec</option>
<option value="PH">Peh\u010devo</option>
<option value="PN">Plasnica</option>
<option value="PP">Prilep</option>
<option value="PT">Probi\u0161tip</option>
<option value="RV">Radovi\u0161</option>
<option value="RN">Rankovce</option>
<option value="RE">Resen</option>
<option value="RO">Rosoman</option>
<option value="AJ">Saraj</option>
<option value="SL">Sveti Nikole</option>
<option value="SS">Sopi\u0161te</option>
<option value="SD">Star Dojran</option>
<option value="NA">Staro Nagori\u010dane</option>
<option value="UG">Struga</option>
<option value="RU">Strumica</option>
<option value="SU">Studeni\u010dani</option>
<option value="TR">Tearce</option>
<option value="ET">Tetovo</option>
<option value="CE">Centar</option>
<option value="CZ">Centar-\u017dupa</option>
<option value="CI">\u010cair</option>
<option value="CA">\u010ca\u0161ka</option>
<option value="CH">\u010ce\u0161inovo-Oble\u0161evo</option>
<option value="CS">\u010cu\u010der-Sandevo</option>
<option value="ST">\u0160tip</option>
<option value="SO">\u0160uto Orizari</option>
</select>'''
        self.assertHTMLEqual(f.render('municipality', 'DL' ), out)

    def test_UMCNField(self):
        error_invalid = [u'This field should contain exactly 13 digits.']
        error_checksum = [u'The UMCN is not valid.']
        error_date =  [u'The first 7 digits of the UMCN '
                        'must represent a valid past date.']
        valid = {
            '2402983450006': '2402983450006',
            '2803984430038': '2803984430038',
            '1909982045004': '1909982045004',
        }
        invalid = {
            '240298345': error_invalid,
            'abcdefghj': error_invalid,
            '2402082450006': error_date,
            '3002982450006': error_date,
            '2402983450007': error_checksum,
            '2402982450006': error_checksum,
        }
        self.assertFieldOutput(UMCNField, valid, invalid)



"""
JP-specific Form helpers
"""

from __future__ import absolute_import

from django.contrib.localflavor.jp.jp_prefectures import JP_PREFECTURES
from django.forms.fields import RegexField, Select
from django.utils.translation import ugettext_lazy as _


class JPPostalCodeField(RegexField):
    """
    A form field that validates its input is a Japanese postcode.

    Accepts 7 digits, with or without a hyphen.
    """
    default_error_messages = {
        'invalid': _('Enter a postal code in the format XXXXXXX or XXX-XXXX.'),
    }

    def __init__(self, max_length=None, min_length=None, *args, **kwargs):
        super(JPPostalCodeField, self).__init__(r'^\d{3}-\d{4}$|^\d{7}$',
            max_length, min_length, *args, **kwargs)

    def clean(self, value):
        """
        Validates the input and returns a string that contains only numbers.
        Returns an empty string for empty values.
        """
        v = super(JPPostalCodeField, self).clean(value)
        return v.replace('-', '')

class JPPrefectureSelect(Select):
    """
    A Select widget that uses a list of Japanese prefectures as its choices.
    """
    def __init__(self, attrs=None):
        super(JPPrefectureSelect, self).__init__(attrs, choices=JP_PREFECTURES)



# -*- coding: utf-8 -*-
"""
An alphabetical list of Finnish municipalities for use as `choices` in a
formfield.

This exists in this standalone file so that it's only imported into memory
when explicitly needed.
"""

MUNICIPALITY_CHOICES = (
    ('akaa', u"Akaa"),
    ('alajarvi', u"Alajärvi"),
    ('alavieska', u"Alavieska"),
    ('alavus', u"Alavus"),
    ('artjarvi', u"Artjärvi"),
    ('asikkala', u"Asikkala"),
    ('askola', u"Askola"),
    ('aura', u"Aura"),
    ('brando', u"Brändö"),
    ('eckero', u"Eckerö"),
    ('enonkoski', u"Enonkoski"),
    ('enontekio', u"Enontekiö"),
    ('espoo', u"Espoo"),
    ('eura', u"Eura"),
    ('eurajoki', u"Eurajoki"),
    ('evijarvi', u"Evijärvi"),
    ('finstrom', u"Finström"),
    ('forssa', u"Forssa"),
    ('foglo', u"Föglö"),
    ('geta', u"Geta"),
    ('haapajarvi', u"Haapajärvi"),
    ('haapavesi', u"Haapavesi"),
    ('hailuoto', u"Hailuoto"),
    ('halsua', u"Halsua"),
    ('hamina', u"Hamina"),
    ('hammarland', u"Hammarland"),
    ('hankasalmi', u"Hankasalmi"),
    ('hanko', u"Hanko"),
    ('harjavalta', u"Harjavalta"),
    ('hartola', u"Hartola"),
    ('hattula', u"Hattula"),
    ('haukipudas', u"Haukipudas"),
    ('hausjarvi', u"Hausjärvi"),
    ('heinola', u"Heinola"),
    ('heinavesi', u"Heinävesi"),
    ('helsinki', u"Helsinki"),
    ('hirvensalmi', u"Hirvensalmi"),
    ('hollola', u"Hollola"),
    ('honkajoki', u"Honkajoki"),
    ('huittinen', u"Huittinen"),
    ('humppila', u"Humppila"),
    ('hyrynsalmi', u"Hyrynsalmi"),
    ('hyvinkaa', u"Hyvinkää"),
    ('hameenkoski', u"Hämeenkoski"),
    ('hameenkyro', u"Hämeenkyrö"),
    ('hameenlinna', u"Hämeenlinna"),
    ('ii', u"Ii"),
    ('iisalmi', u"Iisalmi"),
    ('iitti', u"Iitti"),
    ('ikaalinen', u"Ikaalinen"),
    ('ilmajoki', u"Ilmajoki"),
    ('ilomantsi', u"Ilomantsi"),
    ('imatra', u"Imatra"),
    ('inari', u"Inari"),
    ('inkoo', u"Inkoo"),
    ('isojoki', u"Isojoki"),
    ('isokyro', u"Isokyrö"),
    ('jalasjarvi', u"Jalasjärvi"),
    ('janakkala', u"Janakkala"),
    ('joensuu', u"Joensuu"),
    ('jokioinen', u"Jokioinen"),
    ('jomala', u"Jomala"),
    ('joroinen', u"Joroinen"),
    ('joutsa', u"Joutsa"),
    ('juankoski', u"Juankoski"),
    ('juuka', u"Juuka"),
    ('juupajoki', u"Juupajoki"),
    ('juva', u"Juva"),
    ('jyvaskyla', u"Jyväskylä"),
    ('jamijarvi', u"Jämijärvi"),
    ('jamsa', u"Jämsä"),
    ('jarvenpaa', u"Järvenpää"),
    ('kaarina', u"Kaarina"),
    ('kaavi', u"Kaavi"),
    ('kajaani', u"Kajaani"),
    ('kalajoki', u"Kalajoki"),
    ('kangasala', u"Kangasala"),
    ('kangasniemi', u"Kangasniemi"),
    ('kankaanpaa', u"Kankaanpää"),
    ('kannonkoski', u"Kannonkoski"),
    ('kannus', u"Kannus"),
    ('karijoki', u"Karijoki"),
    ('karjalohja', u"Karjalohja"),
    ('karkkila', u"Karkkila"),
    ('karstula', u"Karstula"),
    ('karttula', u"Karttula"),
    ('karvia', u"Karvia"),
    ('kaskinen', u"Kaskinen"),
    ('kauhajoki', u"Kauhajoki"),
    ('kauhava', u"Kauhava"),
    ('kauniainen', u"Kauniainen"),
    ('kaustinen', u"Kaustinen"),
    ('keitele', u"Keitele"),
    ('kemi', u"Kemi"),
    ('kemijarvi', u"Kemijärvi"),
    ('keminmaa', u"Keminmaa"),
    ('kemionsaari', u"Kemiönsaari"),
    ('kempele', u"Kempele"),
    ('kerava', u"Kerava"),
    ('kerimaki', u"Kerimäki"),
    ('kesalahti', u"Kesälahti"),
    ('keuruu', u"Keuruu"),
    ('kihnio', u"Kihniö"),
    ('kiikoinen', u"Kiikoinen"),
    ('kiiminki', u"Kiiminki"),
    ('kinnula', u"Kinnula"),
    ('kirkkonummi', u"Kirkkonummi"),
    ('kitee', u"Kitee"),
    ('kittila', u"Kittilä"),
    ('kiuruvesi', u"Kiuruvesi"),
    ('kivijarvi', u"Kivijärvi"),
    ('kokemaki', u"Kokemäki"),
    ('kokkola', u"Kokkola"),
    ('kolari', u"Kolari"),
    ('konnevesi', u"Konnevesi"),
    ('kontiolahti', u"Kontiolahti"),
    ('korsnas', u"Korsnäs"),
    ('koskitl', u"Koski Tl"),
    ('kotka', u"Kotka"),
    ('kouvola', u"Kouvola"),
    ('kristiinankaupunki', u"Kristiinankaupunki"),
    ('kruunupyy', u"Kruunupyy"),
    ('kuhmalahti', u"Kuhmalahti"),
    ('kuhmo', u"Kuhmo"),
    ('kuhmoinen', u"Kuhmoinen"),
    ('kumlinge', u"Kumlinge"),
    ('kuopio', u"Kuopio"),
    ('kuortane', u"Kuortane"),
    ('kurikka', u"Kurikka"),
    ('kustavi', u"Kustavi"),
    ('kuusamo', u"Kuusamo"),
    ('kylmakoski', u"Kylmäkoski"),
    ('kyyjarvi', u"Kyyjärvi"),
    ('karkola', u"Kärkölä"),
    ('karsamaki', u"Kärsämäki"),
    ('kokar', u"Kökar"),
    ('koylio', u"Köyliö"),
    ('lahti', u"Lahti"),
    ('laihia', u"Laihia"),
    ('laitila', u"Laitila"),
    ('lapinjarvi', u"Lapinjärvi"),
    ('lapinlahti', u"Lapinlahti"),
    ('lappajarvi', u"Lappajärvi"),
    ('lappeenranta', u"Lappeenranta"),
    ('lapua', u"Lapua"),
    ('laukaa', u"Laukaa"),
    ('lavia', u"Lavia"),
    ('lemi', u"Lemi"),
    ('lemland', u"Lemland"),
    ('lempaala', u"Lempäälä"),
    ('leppavirta', u"Leppävirta"),
    ('lestijarvi', u"Lestijärvi"),
    ('lieksa', u"Lieksa"),
    ('lieto', u"Lieto"),
    ('liminka', u"Liminka"),
    ('liperi', u"Liperi"),
    ('lohja', u"Lohja"),
    ('loimaa', u"Loimaa"),
    ('loppi', u"Loppi"),
    ('loviisa', u"Loviisa"),
    ('luhanka', u"Luhanka"),
    ('lumijoki', u"Lumijoki"),
    ('lumparland', u"Lumparland"),
    ('luoto', u"Luoto"),
    ('luumaki', u"Luumäki"),
    ('luvia', u"Luvia"),
    ('lansi-turunmaa', u"Länsi-Turunmaa"),
    ('maalahti', u"Maalahti"),
    ('maaninka', u"Maaninka"),
    ('maarianhamina', u"Maarianhamina"),
    ('marttila', u"Marttila"),
    ('masku', u"Masku"),
    ('merijarvi', u"Merijärvi"),
    ('merikarvia', u"Merikarvia"),
    ('miehikkala', u"Miehikkälä"),
    ('mikkeli', u"Mikkeli"),
    ('muhos', u"Muhos"),
    ('multia', u"Multia"),
    ('muonio', u"Muonio"),
    ('mustasaari', u"Mustasaari"),
    ('muurame', u"Muurame"),
    ('mynamaki', u"Mynämäki"),
    ('myrskyla', u"Myrskylä"),
    ('mantsala', u"Mäntsälä"),
    ('mantta-vilppula', u"Mänttä-Vilppula"),
    ('mantyharju', u"Mäntyharju"),
    ('naantali', u"Naantali"),
    ('nakkila', u"Nakkila"),
    ('nastola', u"Nastola"),
    ('nilsia', u"Nilsiä"),
    ('nivala', u"Nivala"),
    ('nokia', u"Nokia"),
    ('nousiainen', u"Nousiainen"),
    ('nummi-pusula', u"Nummi-Pusula"),
    ('nurmes', u"Nurmes"),
    ('nurmijarvi', u"Nurmijärvi"),
    ('narpio', u"Närpiö"),
    ('oravainen', u"Oravainen"),
    ('orimattila', u"Orimattila"),
    ('oripaa', u"Oripää"),
    ('orivesi', u"Orivesi"),
    ('oulainen', u"Oulainen"),
    ('oulu', u"Oulu"),
    ('oulunsalo', u"Oulunsalo"),
    ('outokumpu', u"Outokumpu"),
    ('padasjoki', u"Padasjoki"),
    ('paimio', u"Paimio"),
    ('paltamo', u"Paltamo"),
    ('parikkala', u"Parikkala"),
    ('parkano', u"Parkano"),
    ('pedersore', u"Pedersöre"),
    ('pelkosenniemi', u"Pelkosenniemi"),
    ('pello', u"Pello"),
    ('perho', u"Perho"),
    ('pertunmaa', u"Pertunmaa"),
    ('petajavesi', u"Petäjävesi"),
    ('pieksamaki', u"Pieksämäki"),
    ('pielavesi', u"Pielavesi"),
    ('pietarsaari', u"Pietarsaari"),
    ('pihtipudas', u"Pihtipudas"),
    ('pirkkala', u"Pirkkala"),
    ('polvijarvi', u"Polvijärvi"),
    ('pomarkku', u"Pomarkku"),
    ('pori', u"Pori"),
    ('pornainen', u"Pornainen"),
    ('porvoo', u"Porvoo"),
    ('posio', u"Posio"),
    ('pudasjarvi', u"Pudasjärvi"),
    ('pukkila', u"Pukkila"),
    ('punkaharju', u"Punkaharju"),
    ('punkalaidun', u"Punkalaidun"),
    ('puolanka', u"Puolanka"),
    ('puumala', u"Puumala"),
    ('pyhtaa', u"Pyhtää"),
    ('pyhajoki', u"Pyhäjoki"),
    ('pyhajarvi', u"Pyhäjärvi"),
    ('pyhanta', u"Pyhäntä"),
    ('pyharanta', u"Pyhäranta"),
    ('palkane', u"Pälkäne"),
    ('poytya', u"Pöytyä"),
    ('raahe', u"Raahe"),
    ('raasepori', u"Raasepori"),
    ('raisio', u"Raisio"),
    ('rantasalmi', u"Rantasalmi"),
    ('ranua', u"Ranua"),
    ('rauma', u"Rauma"),
    ('rautalampi', u"Rautalampi"),
    ('rautavaara', u"Rautavaara"),
    ('rautjarvi', u"Rautjärvi"),
    ('reisjarvi', u"Reisjärvi"),
    ('riihimaki', u"Riihimäki"),
    ('ristiina', u"Ristiina"),
    ('ristijarvi', u"Ristijärvi"),
    ('rovaniemi', u"Rovaniemi"),
    ('ruokolahti', u"Ruokolahti"),
    ('ruovesi', u"Ruovesi"),
    ('rusko', u"Rusko"),
    ('raakkyla', u"Rääkkylä"),
    ('saarijarvi', u"Saarijärvi"),
    ('salla', u"Salla"),
    ('salo', u"Salo"),
    ('saltvik', u"Saltvik"),
    ('sastamala', u"Sastamala"),
    ('sauvo', u"Sauvo"),
    ('savitaipale', u"Savitaipale"),
    ('savonlinna', u"Savonlinna"),
    ('savukoski', u"Savukoski"),
    ('seinajoki', u"Seinäjoki"),
    ('sievi', u"Sievi"),
    ('siikainen', u"Siikainen"),
    ('siikajoki', u"Siikajoki"),
    ('siikalatva', u"Siikalatva"),
    ('siilinjarvi', u"Siilinjärvi"),
    ('simo', u"Simo"),
    ('sipoo', u"Sipoo"),
    ('siuntio', u"Siuntio"),
    ('sodankyla', u"Sodankylä"),
    ('soini', u"Soini"),
    ('somero', u"Somero"),
    ('sonkajarvi', u"Sonkajärvi"),
    ('sotkamo', u"Sotkamo"),
    ('sottunga', u"Sottunga"),
    ('sulkava', u"Sulkava"),
    ('sund', u"Sund"),
    ('suomenniemi', u"Suomenniemi"),
    ('suomussalmi', u"Suomussalmi"),
    ('suonenjoki', u"Suonenjoki"),
    ('sysma', u"Sysmä"),
    ('sakyla', u"Säkylä"),
    ('taipalsaari', u"Taipalsaari"),
    ('taivalkoski', u"Taivalkoski"),
    ('taivassalo', u"Taivassalo"),
    ('tammela', u"Tammela"),
    ('tampere', u"Tampere"),
    ('tarvasjoki', u"Tarvasjoki"),
    ('tervo', u"Tervo"),
    ('tervola', u"Tervola"),
    ('teuva', u"Teuva"),
    ('tohmajarvi', u"Tohmajärvi"),
    ('toholampi', u"Toholampi"),
    ('toivakka', u"Toivakka"),
    ('tornio', u"Tornio"),
    ('turku', u"Turku"),
    ('tuusniemi', u"Tuusniemi"),
    ('tuusula', u"Tuusula"),
    ('tyrnava', u"Tyrnävä"),
    ('toysa', u"Töysä"),
    ('ulvila', u"Ulvila"),
    ('urjala', u"Urjala"),
    ('utajarvi', u"Utajärvi"),
    ('utsjoki', u"Utsjoki"),
    ('uurainen', u"Uurainen"),
    ('uusikaarlepyy', u"Uusikaarlepyy"),
    ('uusikaupunki', u"Uusikaupunki"),
    ('vaala', u"Vaala"),
    ('vaasa', u"Vaasa"),
    ('valkeakoski', u"Valkeakoski"),
    ('valtimo', u"Valtimo"),
    ('vantaa', u"Vantaa"),
    ('varkaus', u"Varkaus"),
    ('varpaisjarvi', u"Varpaisjärvi"),
    ('vehmaa', u"Vehmaa"),
    ('vesanto', u"Vesanto"),
    ('vesilahti', u"Vesilahti"),
    ('veteli', u"Veteli"),
    ('vierema', u"Vieremä"),
    ('vihanti', u"Vihanti"),
    ('vihti', u"Vihti"),
    ('viitasaari', u"Viitasaari"),
    ('vimpeli', u"Vimpeli"),
    ('virolahti', u"Virolahti"),
    ('virrat', u"Virrat"),
    ('vardo', u"Vårdö"),
    ('vahakyro', u"Vähäkyrö"),
    ('voyri-maksamaa', u"Vöyri-Maksamaa"),
    ('yli-ii', u"Yli-Ii"),
    ('ylitornio', u"Ylitornio"),
    ('ylivieska', u"Ylivieska"),
    ('ylojarvi', u"Ylöjärvi"),
    ('ypaja', u"Ypäjä"),
    ('ahtari', u"Ähtäri"),
    ('aanekoski', u"Äänekoski")
)


from __future__ import absolute_import

import os
import shutil

from django.core.files import File
from django.core.files.images import ImageFile
from django.test import TestCase

from .models import (Image, Person, PersonWithHeight, PersonWithHeightAndWidth,
    PersonDimensionsFirst, PersonTwoImages, TestImageFieldFile)


# If PIL available, do these tests.
if Image:

    from .models import temp_storage_dir


    class ImageFieldTestMixin(object):
        """
        Mixin class to provide common functionality to ImageField test classes.
        """

        # Person model to use for tests.
        PersonModel = PersonWithHeightAndWidth
        # File class to use for file instances.
        File = ImageFile

        def setUp(self):
            """
            Creates a pristine temp directory (or deletes and recreates if it
            already exists) that the model uses as its storage directory.

            Sets up two ImageFile instances for use in tests.
            """
            if os.path.exists(temp_storage_dir):
                shutil.rmtree(temp_storage_dir)
            os.mkdir(temp_storage_dir)

            file_path1 = os.path.join(os.path.dirname(__file__), "4x8.png")
            self.file1 = self.File(open(file_path1, 'rb'))

            file_path2 = os.path.join(os.path.dirname(__file__), "8x4.png")
            self.file2 = self.File(open(file_path2, 'rb'))

        def tearDown(self):
            """
            Removes temp directory and all its contents.
            """
            shutil.rmtree(temp_storage_dir)

        def check_dimensions(self, instance, width, height,
                             field_name='mugshot'):
            """
            Asserts that the given width and height values match both the
            field's height and width attributes and the height and width fields
            (if defined) the image field is caching to.

            Note, this method will check for dimension fields named by adding
            "_width" or "_height" to the name of the ImageField.  So, the
            models used in these tests must have their fields named
            accordingly.

            By default, we check the field named "mugshot", but this can be
            specified by passing the field_name parameter.
            """
            field = getattr(instance, field_name)
            # Check height/width attributes of field.
            if width is None and height is None:
                self.assertRaises(ValueError, getattr, field, 'width')
                self.assertRaises(ValueError, getattr, field, 'height')
            else:
                self.assertEqual(field.width, width)
                self.assertEqual(field.height, height)

            # Check height/width fields of model, if defined.
            width_field_name = field_name + '_width'
            if hasattr(instance, width_field_name):
                self.assertEqual(getattr(instance, width_field_name), width)
            height_field_name = field_name + '_height'
            if hasattr(instance, height_field_name):
                self.assertEqual(getattr(instance, height_field_name), height)


    class ImageFieldTests(ImageFieldTestMixin, TestCase):
        """
        Tests for ImageField that don't need to be run with each of the
        different test model classes.
        """

        def test_equal_notequal_hash(self):
            """
            Bug #9786: Ensure '==' and '!=' work correctly.
            Bug #9508: make sure hash() works as expected (equal items must
            hash to the same value).
            """
            # Create two Persons with different mugshots.
            p1 = self.PersonModel(name="Joe")
            p1.mugshot.save("mug", self.file1)
            p2 = self.PersonModel(name="Bob")
            p2.mugshot.save("mug", self.file2)
            self.assertEqual(p1.mugshot == p2.mugshot, False)
            self.assertEqual(p1.mugshot != p2.mugshot, True)

            # Test again with an instance fetched from the db.
            p1_db = self.PersonModel.objects.get(name="Joe")
            self.assertEqual(p1_db.mugshot == p2.mugshot, False)
            self.assertEqual(p1_db.mugshot != p2.mugshot, True)

            # Instance from db should match the local instance.
            self.assertEqual(p1_db.mugshot == p1.mugshot, True)
            self.assertEqual(hash(p1_db.mugshot), hash(p1.mugshot))
            self.assertEqual(p1_db.mugshot != p1.mugshot, False)

        def test_instantiate_missing(self):
            """
            If the underlying file is unavailable, still create instantiate the
            object without error.
            """
            p = self.PersonModel(name="Joan")
            p.mugshot.save("shot", self.file1)
            p = self.PersonModel.objects.get(name="Joan")
            path = p.mugshot.path
            shutil.move(path, path + '.moved')
            p2 = self.PersonModel.objects.get(name="Joan")

        def test_delete_when_missing(self):
            """
            Bug #8175: correctly delete an object where the file no longer
            exists on the file system.
            """
            p = self.PersonModel(name="Fred")
            p.mugshot.save("shot", self.file1)
            os.remove(p.mugshot.path)
            p.delete()

        def test_size_method(self):
            """
            Bug #8534: FileField.size should not leave the file open.
            """
            p = self.PersonModel(name="Joan")
            p.mugshot.save("shot", self.file1)

            # Get a "clean" model instance
            p = self.PersonModel.objects.get(name="Joan")
            # It won't have an opened file.
            self.assertEqual(p.mugshot.closed, True)

            # After asking for the size, the file should still be closed.
            _ = p.mugshot.size
            self.assertEqual(p.mugshot.closed, True)

        def test_pickle(self):
            """
            Tests that ImageField can be pickled, unpickled, and that the
            image of the unpickled version is the same as the original.
            """
            import pickle

            p = Person(name="Joe")
            p.mugshot.save("mug", self.file1)
            dump = pickle.dumps(p)

            p2 = Person(name="Bob")
            p2.mugshot = self.file1

            loaded_p = pickle.loads(dump)
            self.assertEqual(p.mugshot, loaded_p.mugshot)


    class ImageFieldTwoDimensionsTests(ImageFieldTestMixin, TestCase):
        """
        Tests behavior of an ImageField and its dimensions fields.
        """

        def test_constructor(self):
            """
            Tests assigning an image field through the model's constructor.
            """
            p = self.PersonModel(name='Joe', mugshot=self.file1)
            self.check_dimensions(p, 4, 8)
            p.save()
            self.check_dimensions(p, 4, 8)

        def test_image_after_constructor(self):
            """
            Tests behavior when image is not passed in constructor.
            """
            p = self.PersonModel(name='Joe')
            # TestImageField value will default to being an instance of its
            # attr_class, a  TestImageFieldFile, with name == None, which will
            # cause it to evaluate as False.
            self.assertEqual(isinstance(p.mugshot, TestImageFieldFile), True)
            self.assertEqual(bool(p.mugshot), False)

            # Test setting a fresh created model instance.
            p = self.PersonModel(name='Joe')
            p.mugshot = self.file1
            self.check_dimensions(p, 4, 8)

        def test_create(self):
            """
            Tests assigning an image in Manager.create().
            """
            p = self.PersonModel.objects.create(name='Joe', mugshot=self.file1)
            self.check_dimensions(p, 4, 8)

        def test_default_value(self):
            """
            Tests that the default value for an ImageField is an instance of
            the field's attr_class (TestImageFieldFile in this case) with no
            name (name set to None).
            """
            p = self.PersonModel()
            self.assertEqual(isinstance(p.mugshot, TestImageFieldFile), True)
            self.assertEqual(bool(p.mugshot), False)

        def test_assignment_to_None(self):
            """
            Tests that assigning ImageField to None clears dimensions.
            """
            p = self.PersonModel(name='Joe', mugshot=self.file1)
            self.check_dimensions(p, 4, 8)

            # If image assigned to None, dimension fields should be cleared.
            p.mugshot = None
            self.check_dimensions(p, None, None)

            p.mugshot = self.file2
            self.check_dimensions(p, 8, 4)

        def test_field_save_and_delete_methods(self):
            """
            Tests assignment using the field's save method and deletion using
            the field's delete method.
            """
            p = self.PersonModel(name='Joe')
            p.mugshot.save("mug", self.file1)
            self.check_dimensions(p, 4, 8)

            # A new file should update dimensions.
            p.mugshot.save("mug", self.file2)
            self.check_dimensions(p, 8, 4)

            # Field and dimensions should be cleared after a delete.
            p.mugshot.delete(save=False)
            self.assertEqual(p.mugshot, None)
            self.check_dimensions(p, None, None)

        def test_dimensions(self):
            """
            Checks that dimensions are updated correctly in various situations.
            """
            p = self.PersonModel(name='Joe')

            # Dimensions should get set if file is saved.
            p.mugshot.save("mug", self.file1)
            self.check_dimensions(p, 4, 8)

            # Test dimensions after fetching from database.
            p = self.PersonModel.objects.get(name='Joe')
            # Bug 11084: Dimensions should not get recalculated if file is
            # coming from the database.  We test this by checking if the file
            # was opened.
            self.assertEqual(p.mugshot.was_opened, False)
            self.check_dimensions(p, 4, 8)
            # After checking dimensions on the image field, the file will have
            # opened.
            self.assertEqual(p.mugshot.was_opened, True)
            # Dimensions should now be cached, and if we reset was_opened and
            # check dimensions again, the file should not have opened.
            p.mugshot.was_opened = False
            self.check_dimensions(p, 4, 8)
            self.assertEqual(p.mugshot.was_opened, False)

            # If we assign a new image to the instance, the dimensions should
            # update.
            p.mugshot = self.file2
            self.check_dimensions(p, 8, 4)
            # Dimensions were recalculated, and hence file should have opened.
            self.assertEqual(p.mugshot.was_opened, True)


    class ImageFieldNoDimensionsTests(ImageFieldTwoDimensionsTests):
        """
        Tests behavior of an ImageField with no dimension fields.
        """

        PersonModel = Person


    class ImageFieldOneDimensionTests(ImageFieldTwoDimensionsTests):
        """
        Tests behavior of an ImageField with one dimensions field.
        """

        PersonModel = PersonWithHeight


    class ImageFieldDimensionsFirstTests(ImageFieldTwoDimensionsTests):
        """
        Tests behavior of an ImageField where the dimensions fields are
        defined before the ImageField.
        """

        PersonModel = PersonDimensionsFirst


    class ImageFieldUsingFileTests(ImageFieldTwoDimensionsTests):
        """
        Tests behavior of an ImageField when assigning it a File instance
        rather than an ImageFile instance.
        """

        PersonModel = PersonDimensionsFirst
        File = File


    class TwoImageFieldTests(ImageFieldTestMixin, TestCase):
        """
        Tests a model with two ImageFields.
        """

        PersonModel = PersonTwoImages

        def test_constructor(self):
            p = self.PersonModel(mugshot=self.file1, headshot=self.file2)
            self.check_dimensions(p, 4, 8, 'mugshot')
            self.check_dimensions(p, 8, 4, 'headshot')
            p.save()
            self.check_dimensions(p, 4, 8, 'mugshot')
            self.check_dimensions(p, 8, 4, 'headshot')

        def test_create(self):
            p = self.PersonModel.objects.create(mugshot=self.file1,
                                                headshot=self.file2)
            self.check_dimensions(p, 4, 8)
            self.check_dimensions(p, 8, 4, 'headshot')

        def test_assignment(self):
            p = self.PersonModel()
            self.check_dimensions(p, None, None, 'mugshot')
            self.check_dimensions(p, None, None, 'headshot')

            p.mugshot = self.file1
            self.check_dimensions(p, 4, 8, 'mugshot')
            self.check_dimensions(p, None, None, 'headshot')
            p.headshot = self.file2
            self.check_dimensions(p, 4, 8, 'mugshot')
            self.check_dimensions(p, 8, 4, 'headshot')

            # Clear the ImageFields one at a time.
            p.mugshot = None
            self.check_dimensions(p, None, None, 'mugshot')
            self.check_dimensions(p, 8, 4, 'headshot')
            p.headshot = None
            self.check_dimensions(p, None, None, 'mugshot')
            self.check_dimensions(p, None, None, 'headshot')

        def test_field_save_and_delete_methods(self):
            p = self.PersonModel(name='Joe')
            p.mugshot.save("mug", self.file1)
            self.check_dimensions(p, 4, 8, 'mugshot')
            self.check_dimensions(p, None, None, 'headshot')
            p.headshot.save("head", self.file2)
            self.check_dimensions(p, 4, 8, 'mugshot')
            self.check_dimensions(p, 8, 4, 'headshot')

            # We can use save=True when deleting the image field with null=True
            # dimension fields and the other field has an image.
            p.headshot.delete(save=True)
            self.check_dimensions(p, 4, 8, 'mugshot')
            self.check_dimensions(p, None, None, 'headshot')
            p.mugshot.delete(save=False)
            self.check_dimensions(p, None, None, 'mugshot')
            self.check_dimensions(p, None, None, 'headshot')

        def test_dimensions(self):
            """
            Checks that dimensions are updated correctly in various situations.
            """
            p = self.PersonModel(name='Joe')

            # Dimensions should get set for the saved file.
            p.mugshot.save("mug", self.file1)
            p.headshot.save("head", self.file2)
            self.check_dimensions(p, 4, 8, 'mugshot')
            self.check_dimensions(p, 8, 4, 'headshot')

            # Test dimensions after fetching from database.
            p = self.PersonModel.objects.get(name='Joe')
            # Bug 11084: Dimensions should not get recalculated if file is
            # coming from the database.  We test this by checking if the file
            # was opened.
            self.assertEqual(p.mugshot.was_opened, False)
            self.assertEqual(p.headshot.was_opened, False)
            self.check_dimensions(p, 4, 8,'mugshot')
            self.check_dimensions(p, 8, 4, 'headshot')
            # After checking dimensions on the image fields, the files will
            # have been opened.
            self.assertEqual(p.mugshot.was_opened, True)
            self.assertEqual(p.headshot.was_opened, True)
            # Dimensions should now be cached, and if we reset was_opened and
            # check dimensions again, the file should not have opened.
            p.mugshot.was_opened = False
            p.headshot.was_opened = False
            self.check_dimensions(p, 4, 8,'mugshot')
            self.check_dimensions(p, 8, 4, 'headshot')
            self.assertEqual(p.mugshot.was_opened, False)
            self.assertEqual(p.headshot.was_opened, False)

            # If we assign a new image to the instance, the dimensions should
            # update.
            p.mugshot = self.file2
            p.headshot = self.file1
            self.check_dimensions(p, 8, 4, 'mugshot')
            self.check_dimensions(p, 4, 8, 'headshot')
            # Dimensions were recalculated, and hence file should have opened.
            self.assertEqual(p.mugshot.was_opened, True)
            self.assertEqual(p.headshot.was_opened, True)
